# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import sys
from typing import Optional

from PyQt5 import QtCore, QtWidgets
from PyQt5.QtCore import QPoint
from PyQt5.QtGui import QKeyEvent, QPixmap
from PyQt5.QtWidgets import QApplication, QMainWindow, QWidget

from tikka.domains.application import Application
from tikka.domains.entities.constants import DATA_PATH
from tikka.domains.entities.entry_point import EntryPoint
from tikka.domains.entities.events import (
    ConnectionsEvent,
    CurrencyEvent,
    EntryPointsEvent,
    NodesEvent,
)
from tikka.slots.pyqt.entities.constants import (
    ICON_NETWORK_CONNECTED,
    ICON_NETWORK_DISCONNECTED,
)
from tikka.slots.pyqt.entities.worker import AsyncQWorker
from tikka.slots.pyqt.models.nodes import NodesTableModel
from tikka.slots.pyqt.resources.gui.widgets.nodes_rc import Ui_NodesWidget
from tikka.slots.pyqt.widgets.node_menu import NodePopupMenu


class NodesWidget(QWidget, Ui_NodesWidget):
    """
    NodesWidget class
    """

    def __init__(
        self,
        application: Application,
        parent: Optional[QWidget] = None,
    ) -> None:
        """
        Init NodesWidget instance

        :param application: Application instance
        :param parent: QWidget instance
        """
        super().__init__(parent=parent)
        self.setupUi(self)

        self.application = application
        self._ = self.application.translator.gettext

        self.connect_button_text_when_connected = self._("Reconnect")
        self.connect_button_text_when_disconnected = self._("Connect")

        self.init_entry_points()

        self.nodes_table_model = NodesTableModel(self.application)
        self.nodesTableView.setModel(self.nodes_table_model)
        self.nodesTableView.resizeColumnsToContents()
        self.nodesTableView.resizeRowsToContents()

        self.nodesTableView.customContextMenuRequested.connect(self.on_context_menu)

        if self.application.connections.is_connected():
            self._on_entry_point_connected()
        else:
            self._on_entry_point_disconnected()

        # events
        self.entryPointComboBox.keyPressEvent = (
            self.on_entry_point_combobox_key_press_event
        )
        self.entryPointComboBox.activated.connect(
            self.on_entry_point_combobox_index_changed
        )
        self.connectButton.clicked.connect(self._on_connect_button_clicked_event)
        self.refreshNodesButton.clicked.connect(
            self._on_refresh_nodes_button_clicked_event
        )

        # subscribe to application events
        self.application.event_dispatcher.add_event_listener(
            CurrencyEvent.EVENT_TYPE_CHANGED, self._on_currency_event
        )
        self.application.event_dispatcher.add_event_listener(
            EntryPointsEvent.EVENT_TYPE_LIST_CHANGED,
            lambda event: self.init_entry_points(),
        )
        self.application.event_dispatcher.add_event_listener(
            NodesEvent.EVENT_TYPE_LIST_CHANGED, self._on_node_list_changed_event
        )
        self.application.event_dispatcher.add_event_listener(
            ConnectionsEvent.EVENT_TYPE_CONNECTED, self._on_entry_point_connected
        )
        self.application.event_dispatcher.add_event_listener(
            ConnectionsEvent.EVENT_TYPE_DISCONNECTED, self._on_entry_point_disconnected
        )

        ##############################
        # ASYNC METHODS
        ##############################
        # Create a QWorker object
        self.network_fetch_current_node_async_qworker = AsyncQWorker(
            self.application.entry_points.network_fetch_current_node
        )
        self.network_fetch_current_node_async_qworker.finished.connect(
            lambda: self.refreshNodesButton.setEnabled(True)
        )

    def init_entry_points(self) -> None:
        """
        Init entry points combobox

        :return:
        """
        self.entryPointComboBox.clear()

        entry_point_urls = [
            entry_point.url for entry_point in self.application.entry_points.list()
        ]
        self.entryPointComboBox.addItems(entry_point_urls)
        # get current entry point url from domain
        current_entry_point_url = self.application.entry_points.get_current_url()
        self.entryPointComboBox.setCurrentIndex(
            entry_point_urls.index(current_entry_point_url)
        )

    def on_entry_point_combobox_key_press_event(self, event: QKeyEvent):
        """
        Triggered when enter is pressed to validate edit in entry point combobox

        :param event: QKeyEvent instance
        :return:
        """
        if event.key() == QtCore.Qt.Key_Return:

            if self.entryPointComboBox.currentText() not in [
                self.entryPointComboBox.itemText(index)
                for index in range(0, self.entryPointComboBox.count())
            ]:
                self.application.entry_points.add(
                    EntryPoint(self.entryPointComboBox.currentText(), None)
                )

                self.application.event_dispatcher.dispatch_event(
                    EntryPointsEvent(EntryPointsEvent.EVENT_TYPE_LIST_CHANGED)
                )
        else:
            QtWidgets.QComboBox.keyPressEvent(self.entryPointComboBox, event)
            # if the key is not return, handle normally

    def on_entry_point_combobox_index_changed(self):
        """
        Triggered when entry point selection is changed

        :return:
        """
        url = self.entryPointComboBox.currentText()
        if url:
            entry_point = self.application.entry_points.get(url)
            if entry_point is None:
                # get the first one
                url = self.entryPointComboBox.itemText(0)
            self.application.entry_points.set_current_url(url)
            self.update_nodes()

    def _on_connect_button_clicked_event(self):
        """
        Triggered when user click on connect button

        :return:
        """
        if self.application.connections.is_connected():
            self.application.connections.disconnect()

        url = self.entryPointComboBox.currentText()
        if url:
            entry_point = self.application.entry_points.get(url)
            if entry_point is not None:
                self.application.connections.connect(entry_point)

    def _on_refresh_nodes_button_clicked_event(self):
        """
        Triggered when user click on refresh nodes button

        :return:
        """
        self.update_nodes()

    def update_nodes(self):
        """
        Update nodes from current entry point in table model and view

        :return:
        """
        # Disable button
        self.refreshNodesButton.setEnabled(False)
        # Start the thread
        self.network_fetch_current_node_async_qworker.start()

    def _on_currency_event(self, _):
        """
        When a currency event is triggered

        :param _: CurrencyEvent instance
        :return:
        """
        self.init_entry_points()

    def _on_node_list_changed_event(self, _):
        """
        When the node list has changed

        :param _: NodesEvent instance
        :return:
        """
        # update model
        self.nodesTableView.model().init_data()
        # resize view
        self.nodesTableView.resizeColumnsToContents()
        self.nodesTableView.resizeRowsToContents()

    def on_context_menu(self, position: QPoint):
        """
        When right button on table view

        :param position: QPoint instance
        :return:
        """
        index = self.nodesTableView.indexAt(position)
        if index.isValid():
            # get selected node
            row = index.row()
            node = self.nodes_table_model.nodes[row]
            # display popup menu at click position
            is_current_node = False
            if len(node.entry_points) > 0:
                entry_point = node.entry_points[0]
                is_current_node = (
                    entry_point.url == self.entryPointComboBox.currentText()
                )
            NodePopupMenu(self.application, node, is_current_node).exec_(
                self.nodesTableView.mapToGlobal(position)
            )

    def _on_entry_point_connected(self, _=None):
        """
        Triggered when entry point is connected

        :return:
        """
        self.connectButton.setText(self.connect_button_text_when_connected)
        self.connectionStatusLabel.setPixmap(QPixmap(ICON_NETWORK_CONNECTED))

    def _on_entry_point_disconnected(self, _=None):
        """
        Triggered when entry point is disconnected

        :return:
        """
        self.connectButton.setText(self.connect_button_text_when_disconnected)
        self.connectionStatusLabel.setPixmap(QPixmap(ICON_NETWORK_DISCONNECTED))


if __name__ == "__main__":
    qapp = QApplication(sys.argv)

    application_ = Application(DATA_PATH)

    main_window = QMainWindow()
    main_window.show()

    main_window.setCentralWidget(NodesWidget(application_, main_window))

    sys.exit(qapp.exec_())



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import copy
import logging
from typing import Optional

from tikka.domains.entities.account import Account
from tikka.domains.entities.events import AccountEvent
from tikka.domains.events import EventDispatcher
from tikka.domains.wallets import Wallets
from tikka.interfaces.adapters.network.accounts import NetworkAccountsInterface
from tikka.interfaces.adapters.repository.accounts import AccountsRepositoryInterface
from tikka.interfaces.adapters.repository.file_wallets import (
    V1FileWalletsRepositoryInterface,
)


class Accounts:

    list: list = []

    """
    Account domain class
    """

    def __init__(
        self,
        repository: AccountsRepositoryInterface,
        network: NetworkAccountsInterface,
        wallets: Wallets,
        file_wallets_repository: V1FileWalletsRepositoryInterface,
        event_dispatcher: EventDispatcher,
    ):
        """
        Init Accounts domain

        :param repository: AccountsRepositoryInterface instance
        :param network: NetworkAccountsInterface instance
        :param wallets: Wallets domain instance
        :param file_wallets_repository: FileWalletsRepository adapter instance
        :param event_dispatcher: EventDispatcher instance
        """
        self.repository = repository
        self.network = network
        self.wallets = wallets
        self.file_wallets_repository = file_wallets_repository
        self.event_dispatcher = event_dispatcher

        # init account list from database
        self.init_list()

    def init_list(self):
        """
        Init accounts from currency database connection

        :return:
        """
        # get accounts from database
        self.list = self.repository.list()

    def add(self, account: Account):
        """
        Add account

        :param account: Account instance
        :return:
        """
        # add account
        self.list.append(account)
        self.repository.add(account)

        # dispatch event
        event = AccountEvent(
            AccountEvent.EVENT_TYPE_ADD,
            copy.copy(account),
        )
        self.event_dispatcher.dispatch_event(event)

    def update(self, account: Account):
        """
        Update account

        :param account: Account instance
        :return:
        """
        self.repository.update(account)

    def get_by_index(self, index: int) -> Account:
        """
        Return account instance from index

        :param index: Index in account list
        :return:
        """
        return self.list[index]

    def get_by_address(self, address: str) -> Optional[Account]:
        """
        Return account instance from address

        :param address: Account address
        :return:
        """
        for account in self.list:
            if account.address == address:
                return account

        return None

    def delete(self, account: Account) -> None:
        """
        Delete account in list and repository

        :param account: Account instance to delete
        :return:
        """
        index = self.list.index(account)
        del self.list[index]
        self.repository.delete(account)
        self.wallets.delete(account.address)

        # dispatch event
        event = AccountEvent(
            AccountEvent.EVENT_TYPE_DELETE,
            copy.copy(account),
        )
        self.event_dispatcher.dispatch_event(event)

    def unlock(self, account: Account, wallet_password: str) -> bool:
        """
        Unlock account if password is OK

        :param account: Account instance
        :param wallet_password: Passphrase
        :return:
        """
        # get keypair from stored wallet
        try:
            keypair = self.wallets.get_keypair(account.address, wallet_password)
        except Exception as exception:
            logging.exception(exception)
            return False
        if keypair is None:
            return False

        # save keypair in account instance
        account.keypair = keypair

        # dispatch event
        event = AccountEvent(
            AccountEvent.EVENT_TYPE_UPDATE,
            copy.copy(account),
        )
        self.event_dispatcher.dispatch_event(event)
        return True

    def lock(self, account: Account):
        """
        Lock account by removing keypair

        :param account: Account instance
        :return:
        """
        account.keypair = None

        # dispatch event
        event = AccountEvent(
            AccountEvent.EVENT_TYPE_UPDATE,
            copy.copy(account),
        )
        self.event_dispatcher.dispatch_event(event)

    def fetch_balance_from_network(self, account: Account) -> None:
        """
        Fetch account balance from current EntryPoint connection

        :param account: Account instance
        :return:
        """
        balance = self.network.get_balance(account.address)
        if balance is not None:
            account.balance = balance



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import sys
from typing import TYPE_CHECKING, Optional

from PyQt5 import QtCore, QtWidgets
from PyQt5.QtGui import QKeyEvent
from PyQt5.QtWidgets import QApplication, QDialog, QLineEdit, QWidget

from tikka.domains.application import Application
from tikka.domains.entities.account import Account
from tikka.domains.entities.constants import DATA_PATH, WALLETS_PASSWORD_LENGTH
from tikka.libs.secret import generate_alphabetic
from tikka.slots.pyqt.resources.gui.windows.wallet_password_change_rc import (
    Ui_PasswordChangeDialog,
)

if TYPE_CHECKING:
    import _


class WalletPasswordChangeWindow(QDialog, Ui_PasswordChangeDialog):
    """
    WalletPasswordChangeWindow class
    """

    def __init__(
        self,
        application: Application,
        account: Account,
        parent: Optional[QWidget] = None,
    ):
        """
        Init password change window

        :param application: Application instance
        :param account: Account instance
        :param parent: QWidget instance
        """
        super().__init__(parent=parent)
        self.setupUi(self)

        self.application = application
        self.account = account
        self.keypair = None
        self._ = self.application.translator.gettext

        # populate fields
        self.addressValueLabel.setText(self.account.address)

        # if account is already unlocked...
        if self.account.keypair is not None:
            # get keypair
            self.keypair = self.account.keypair
            # hide useless password fields
            self.passwordLabel.hide()
            self.passwordLineEdit.hide()
            self.showButton.hide()
        else:
            # disable Ok button
            self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)

        # events
        self.showButton.clicked.connect(self.on_show_button_clicked)
        self.passwordLineEdit.keyPressEvent = self._on_password_line_edit_keypress_event
        self.changeButton.clicked.connect(self._generate_wallet_password)
        self.buttonBox.accepted.connect(self.on_accepted_button)
        self.buttonBox.rejected.connect(self.on_rejected_button)

        self._generate_wallet_password()

    def _generate_wallet_password(self):
        """
        Generate new password for wallet encryption in UI

        :return:
        """
        self.newPasswordLineEdit.setText(generate_alphabetic(WALLETS_PASSWORD_LENGTH))

    def on_show_button_clicked(self):
        """
        Triggered when user click on show button

        :return:
        """
        if self.passwordLineEdit.echoMode() == QLineEdit.Password:
            self.passwordLineEdit.setEchoMode(QLineEdit.Normal)
            self.showButton.setText(self._("Hide"))
        else:
            self.passwordLineEdit.setEchoMode(QLineEdit.Password)
            self.showButton.setText(self._("Show"))

    def _on_password_line_edit_keypress_event(self, event: QKeyEvent):
        """
        Triggered when a key is pressed in the password field

        :return:
        """
        if event.key() == QtCore.Qt.Key_Return:
            self._get_keypair()
        else:
            QtWidgets.QLineEdit.keyPressEvent(self.passwordLineEdit, event)
            # if the key is not return, handle normally

    def _get_keypair(self) -> None:
        """
        Validate fields and get wallet keypair to enabled ok button

        :return:
        """
        password = self.passwordLineEdit.text().strip().upper()
        try:
            self.keypair = self.application.wallets.get_keypair(
                self.account.address, password
            )
        except Exception:
            self.errorLabel.setText(self._("Password is not valid!"))
            self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
            return None

        if not self.keypair:
            self.errorLabel.setText(self._("Password is not valid!"))
            self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
            return None

        self.buttonBox.button(self.buttonBox.Ok).setEnabled(True)
        self.errorLabel.setText("")
        return None

    def on_rejected_button(self) -> None:
        """
        Triggered when user click on cancel button

        :return:
        """
        self.application.accounts.lock(self.account)

    def on_accepted_button(self):
        """
        Triggered when user click on ok button

        :return:
        """
        new_password = self.newPasswordLineEdit.text().strip()

        # create new Wallet instance and update repository
        wallet = self.application.wallets.create(self.keypair, new_password)
        self.application.wallets.update(wallet)

        # close window
        del self.keypair
        self.close()


if __name__ == "__main__":
    qapp = QApplication(sys.argv)
    application_ = Application(DATA_PATH)
    account_ = Account("CUYYUnh7N49WZhs5DULkmqw5Zu5fwsRBmE5LLrUFRpgw")
    WalletPasswordChangeWindow(application_, account_).exec_()



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from dataclasses import dataclass
from typing import ClassVar

from tikka.domains.entities.account import Account
from tikka.interfaces.entities.events import EventInterface


@dataclass
class AccountEvent(EventInterface):
    """
    AccountEvent class
    """

    type: str
    account: Account

    # type ignore required because mypy bug https://github.com/python/mypy/issues/6473
    EVENT_TYPE_ADD: ClassVar[str] = f"{__qualname__}.add"  # type: ignore
    EVENT_TYPE_UPDATE: ClassVar[str] = f"{__qualname__}.update"  # type: ignore
    EVENT_TYPE_DELETE: ClassVar[str] = f"{__qualname__}.delete"  # type: ignore


@dataclass
class CurrencyEvent(EventInterface):
    """
    ConfigEvent class
    """

    type: str
    currency: str

    # type ignore required because mypy bug https://github.com/python/mypy/issues/6473
    EVENT_TYPE_PRE_CHANGE: ClassVar[str] = f"{__qualname__}.pre_change"  # type: ignore
    EVENT_TYPE_CHANGED: ClassVar[str] = f"{__qualname__}.changed"  # type: ignore


@dataclass
class EntryPointsEvent(EventInterface):
    """
    EntryPointsEvent class
    """

    type: str

    # type ignore required because mypy bug https://github.com/python/mypy/issues/6473
    EVENT_TYPE_LIST_CHANGED: ClassVar[str] = f"{__qualname__}.list_changed"  # type: ignore


@dataclass
class NodesEvent(EventInterface):
    """
    NodesEvent class
    """

    type: str

    # type ignore required because mypy bug https://github.com/python/mypy/issues/6473
    EVENT_TYPE_LIST_CHANGED: ClassVar[str] = f"{__qualname__}.list_changed"  # type: ignore


@dataclass
class ConnectionsEvent(EventInterface):
    """
    ConnectionsEvent class
    """

    type: str

    # type ignore required because mypy bug https://github.com/python/mypy/issues/6473
    EVENT_TYPE_CONNECTED: ClassVar[str] = f"{__qualname__}.connected"  # type: ignore
    EVENT_TYPE_DISCONNECTED: ClassVar[str] = f"{__qualname__}.disconnected"  # type: ignore


@dataclass
class UnitEvent(EventInterface):
    """
    UnitEvent class
    """

    type: str

    # type ignore required because mypy bug https://github.com/python/mypy/issues/6473
    EVENT_TYPE_CHANGED: ClassVar[str] = f"{__qualname__}.changed"  # type: ignore


@dataclass
class TransferEvent(EventInterface):
    """
    TransferEvent class
    """

    type: str

    # type ignore required because mypy bug https://github.com/python/mypy/issues/6473
    EVENT_TYPE_SENT: ClassVar[str] = f"{__qualname__}.sent"  # type: ignore



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import sys
import uuid
from typing import Optional

from PyQt5.QtWidgets import QApplication, QMenu, QMessageBox, QWidget

from tikka.domains.application import Application
from tikka.domains.entities.constants import DATA_PATH
from tikka.domains.entities.entry_point import EntryPoint
from tikka.domains.entities.events import EntryPointsEvent
from tikka.domains.entities.node import Node


class NodePopupMenu(QMenu):
    """
    NodePopupMenu class
    """

    def __init__(
        self,
        application: Application,
        node: Node,
        is_current_node: bool,
        parent: Optional[QWidget] = None,
    ):
        """
        Init NodePopupMenu instance

        :param application: Application instance
        :param node: Node instance
        :param is_current_node: True if node is the node of the current selected entry point
        :param parent: QWidget instance
        """
        super().__init__(parent=parent)

        self.application = application
        self.node = node
        self.is_current_node = is_current_node
        self._ = self.application.translator.gettext

        # menu actions
        copy_peer_id_to_clipboard_action = self.addAction(
            self._("Copy peer ID to clipboard")
        )
        copy_peer_id_to_clipboard_action.triggered.connect(
            self.copy_entry_points_to_clipboard
        )
        if not self.is_current_node:
            delete_node_action = self.addAction(self._("Delete entry point"))
            delete_node_action.triggered.connect(self.delete_node)

    def copy_entry_points_to_clipboard(self):
        """
        Copy peer ID to clipboard

        :return:
        """
        clipboard = QApplication.clipboard()
        clipboard.setText(self.node.peer_id)

    def delete_node(self):
        """
        Delete selected node and its entry point

        :return:
        """
        entry_points = self.application.entry_points.list(
            offset=0, limit=1, node_id=self.node.id
        )
        if len(entry_points) > 0:
            entry_point = entry_points[0]
            response_button = self.confirm_delete_node(entry_point)
            if response_button == QMessageBox.Yes:
                self.application.entry_points.delete(entry_point.url)
                self.application.nodes.delete(self.node.id)
                self.application.event_dispatcher.dispatch_event(
                    EntryPointsEvent(EntryPointsEvent.EVENT_TYPE_LIST_CHANGED)
                )

    def confirm_delete_node(
        self, entry_point: EntryPoint
    ) -> QMessageBox.StandardButton:
        """
        Display confirm dialog and return response

        :param entry_point: EntryPoint instance
        :return:
        """
        # display confirm dialog and get response
        custom_question = self._("Delete entry point {}?")
        return QMessageBox.question(
            self,
            self._("Delete entry point"),
            custom_question.format(entry_point.url),
        )


if __name__ == "__main__":
    qapp = QApplication(sys.argv)

    application_ = Application(DATA_PATH)
    node_ = Node(
        uuid.uuid4(),
        "732SSfuwjB7jkt9th1zerGhphs6nknaCBCTozxUcPWPU",
        999,
        "duniter",
        "1.8.1",
    )

    menu = NodePopupMenu(application_, node_, False)
    menu.exec_()

    sys.exit(qapp.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/about.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AboutDialog(object):
    def setupUi(self, AboutDialog):
        AboutDialog.setObjectName("AboutDialog")
        AboutDialog.resize(506, 595)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(AboutDialog)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.logoLabel = QtWidgets.QLabel(AboutDialog)
        self.logoLabel.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.logoLabel.setText("")
        self.logoLabel.setPixmap(QtGui.QPixmap(":/images/logo"))
        self.logoLabel.setScaledContents(False)
        self.logoLabel.setObjectName("logoLabel")
        self.verticalLayout_2.addWidget(self.logoLabel, 0, QtCore.Qt.AlignHCenter)
        self.descriptionLabel = QtWidgets.QLabel(AboutDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.descriptionLabel.setFont(font)
        self.descriptionLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.descriptionLabel.setObjectName("descriptionLabel")
        self.verticalLayout_2.addWidget(self.descriptionLabel)
        self.versionLabel = QtWidgets.QLabel(AboutDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.versionLabel.setFont(font)
        self.versionLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.versionLabel.setObjectName("versionLabel")
        self.verticalLayout_2.addWidget(self.versionLabel)
        self.authorTitleLabel = QtWidgets.QLabel(AboutDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.authorTitleLabel.setFont(font)
        self.authorTitleLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.authorTitleLabel.setObjectName("authorTitleLabel")
        self.verticalLayout_2.addWidget(self.authorTitleLabel)
        self.authorListLabel = QtWidgets.QLabel(AboutDialog)
        self.authorListLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.authorListLabel.setObjectName("authorListLabel")
        self.verticalLayout_2.addWidget(self.authorListLabel)

        self.retranslateUi(AboutDialog)
        QtCore.QMetaObject.connectSlotsByName(AboutDialog)

    def retranslateUi(self, AboutDialog):
        #  _translate = QtCore.QCoreApplication.translate
        AboutDialog.setWindowTitle(_("About"))
        self.descriptionLabel.setText(_("A free software desktop client for Duniter. \n"
"Manage your Ğ1 accounts securely and easily!"))
        self.versionLabel.setText(_("Version x.y.z"))
        self.authorTitleLabel.setText(_("Author"))
        self.authorListLabel.setText(_("Author list"))
import tikka.slots.pyqt.resources.icons.index_rc



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import sys
from pathlib import Path
from typing import TYPE_CHECKING, Optional

from PyQt5 import QtCore, QtWidgets
from PyQt5.QtGui import QKeyEvent
from PyQt5.QtWidgets import (
    QApplication,
    QDialog,
    QFileDialog,
    QLineEdit,
    QMessageBox,
    QWidget,
)
from substrateinterface import Keypair, KeypairType

from tikka.domains.application import Application
from tikka.domains.entities.account import Account
from tikka.domains.entities.constants import DATA_PATH, WALLETS_PASSWORD_LENGTH
from tikka.domains.entities.wallet import V1FileWallet
from tikka.interfaces.adapters.repository.preferences import (
    WALLET_LOAD_DEFAULT_DIRECTORY,
)
from tikka.libs.secret import generate_alphabetic
from tikka.slots.pyqt.resources.gui.windows.v1_wallet_import_rc import (
    Ui_V1WalletImportDialog,
)

if TYPE_CHECKING:
    pass


class V1WalletImportWindow(QDialog, Ui_V1WalletImportDialog):
    """
    V1WalletImportWindow class
    """

    def __init__(
        self,
        application: Application,
        parent: Optional[QWidget] = None,
    ):
        """
        Init V1 import wallet window

        :param application: Application instance
        :param parent: QWidget instance
        """
        super().__init__(parent=parent)
        self.setupUi(self)

        self.application = application
        self.v1_wallet: Optional[V1FileWallet] = None
        self._ = self.application.translator.gettext

        # events
        self.showButton.clicked.connect(self._on_show_button_clicked)
        self.browseFilesButton.clicked.connect(self._on_browse_files_button_clicked)
        self.filePasswordLineEdit.keyPressEvent = (
            self._on_password_line_edit_keypress_event
        )
        self.buttonBox.rejected.connect(self.close)
        self.buttonBox.accepted.connect(self._on_accepted_button)
        self.changeButton.clicked.connect(self._generate_wallet_password)

        self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
        self._generate_wallet_password()

    def _generate_wallet_password(self):
        """
        Generate new password for wallet encryption in UI

        :return:
        """
        self.passwordLineEdit.setText(generate_alphabetic(WALLETS_PASSWORD_LENGTH))

    def _on_show_button_clicked(self):
        """
        Triggered when user click on show button

        :return:
        """
        if self.filePasswordLineEdit.echoMode() == QLineEdit.Password:
            self.filePasswordLineEdit.setEchoMode(QLineEdit.Normal)
            self.showButton.setText(self._("Hide"))
        else:
            self.filePasswordLineEdit.setEchoMode(QLineEdit.Password)
            self.showButton.setText(self._("Show"))

    def _on_browse_files_button_clicked(self):
        """
        Triggered when user click on browse files button

        :return:
        """
        # clear path
        self.pathValueLabel.setText("")

        # open file dialog
        filepath = self.open_file_dialog()
        if filepath is not None:
            # update default dir preference
            self.application.preferences_repository.set(
                WALLET_LOAD_DEFAULT_DIRECTORY,
                str(Path(filepath).expanduser().absolute().parent),
            )
            self.pathValueLabel.setText(filepath)
            self._load_wallet()

    def _load_wallet(self):
        """
        Load wallet file

        :return:
        """
        password = self.filePasswordLineEdit.text().strip()
        try:
            self.v1_wallet = self.application.file_wallets_repository.load(
                self.pathValueLabel.text(), password
            )
        except Exception:
            self.errorLabel.setText(self._("Unable to decrypt file!"))
            self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
            return

        if not self.v1_wallet.signing_key:
            self.errorLabel.setText(self._("File password is not valid!"))
            self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
            return

        # wallet decrypted
        self.v1AddressValueLabel.setText(self.v1_wallet.signing_key.pubkey)
        keypair = Keypair.create_from_seed(
            seed_hex=self.v1_wallet.signing_key.seed.hex(),
            ss58_format=self.application.currencies.get_current().ss58_format,
            crypto_type=KeypairType.ED25519,
        )
        self.addressValueLabel.setText(keypair.ss58_address)
        if self.application.accounts.get_by_address(keypair.ss58_address) is not None:
            self.errorLabel.setText(self._("Account already exists!"))
            self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
        else:
            self.errorLabel.setText("")
            self.buttonBox.button(self.buttonBox.Ok).setEnabled(True)

    def open_file_dialog(self) -> Optional[str]:
        """
        Open file dialog and return the selected filepath or None

        :return:
        """
        default_dir = self.application.preferences_repository.get(
            WALLET_LOAD_DEFAULT_DIRECTORY
        )
        if default_dir is not None:
            default_dir = str(Path(default_dir).expanduser().absolute())
        else:
            default_dir = ""

        result = QFileDialog.getOpenFileName(
            self, self._("Choose wallet file"), default_dir, "EWIF Files (*.dunikey)"
        )
        if result[0] == "":
            return None

        return result[0]

    def _on_password_line_edit_keypress_event(self, event: QKeyEvent):
        """
        Triggered when a key is pressed in the password field

        :return:
        """
        if event.key() == QtCore.Qt.Key_Return:
            self._load_wallet()
        else:
            QtWidgets.QLineEdit.keyPressEvent(self.filePasswordLineEdit, event)
            # if the key is not return, handle normally

    def _on_accepted_button(self):
        """
        Triggered when user click on cancel button

        :return:
        """
        if self.v1_wallet is not None:
            password = self.passwordLineEdit.text()
            keypair = Keypair.create_from_seed(
                seed_hex=self.v1_wallet.signing_key.seed.hex(),
                ss58_format=self.application.currencies.get_current().ss58_format,
                crypto_type=KeypairType.ED25519,
            )
            address = keypair.ss58_address

            wallet = self.application.wallets.get(address)
            if wallet is None:
                # create and store Wallet instance
                wallet = self.application.wallets.create(keypair, password)
                self.application.wallets.add(wallet)
            else:
                # display confirm dialog and get response
                button = QMessageBox.question(
                    self,
                    self._("Change wallet password?"),
                    self._(
                        "A wallet already exists for this account. Change password for {address} wallet?"
                    ).format(address=wallet.address),
                )
                if button == QMessageBox.Yes:
                    # create and store Wallet instance
                    new_wallet = self.application.wallets.create(keypair, password)
                    self.application.wallets.update(new_wallet)

            account = self.application.accounts.get_by_address(address)
            if account is None:
                # create and store Account instance
                account = Account(
                    address,
                    crypto_type=KeypairType.ED25519,
                    _keypair=keypair,
                )
                self.application.accounts.add(account)
            else:
                self.application.accounts.unlock(account, password)
                self.application.accounts.update(account)


if __name__ == "__main__":
    qapp = QApplication(sys.argv)
    application_ = Application(DATA_PATH)
    V1WalletImportWindow(
        application_,
    ).exec_()



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import logging
import sys
from collections import OrderedDict
from typing import Optional

from PyQt5.QtWidgets import QApplication, QDialog, QMessageBox, QWidget
from substrateinterface import Keypair, KeypairType

from tikka.domains.application import Application
from tikka.domains.entities.account import Account
from tikka.domains.entities.constants import DATA_PATH, WALLETS_PASSWORD_LENGTH
from tikka.libs.secret import generate_alphabetic
from tikka.slots.pyqt.resources.gui.windows.account_import_rc import (
    Ui_AccountImportDialog,
)


class AccountImportWindow(QDialog, Ui_AccountImportDialog):
    """
    AccountImportWindow class
    """

    def __init__(self, application: Application, parent: Optional[QWidget] = None):
        """
        Init import account window

        :param application: Application instance
        :param parent: QWidget instance
        """
        super().__init__(parent=parent)
        self.setupUi(self)

        self.application = application
        self._ = self.application.translator.gettext

        # Mnemonic language selector translated
        mnemonic_language_selector = OrderedDict(
            [
                ("en", self._("English")),
                ("fr", self._("French")),
                ("zh-hans", self._("Chinese simplified")),
                ("zh-hant", self._("Chinese traditional")),
                ("it", self._("Italian")),
                ("ja", self._("Japanese")),
                ("ko", self._("Korean")),
                ("es", self._("Spanish")),
            ]
        )
        self.mnemonic_language_codes = list(mnemonic_language_selector.keys())
        for language_name in mnemonic_language_selector.values():
            self.mnemonicLanguageComboBox.addItem(self._(language_name))
        if self.application.config.get("language") == "fr_FR":
            self.mnemonicLanguageComboBox.setCurrentIndex(
                self.mnemonic_language_codes.index("fr")
            )
        elif self.application.config.get("language") == "en_US":
            self.mnemonicLanguageComboBox.setCurrentIndex(
                self.mnemonic_language_codes.index("en")
            )

        # buttons
        self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)

        # events
        self.mnemonicLanguageComboBox.currentIndexChanged.connect(
            self._generate_address
        )
        self.mnemonicLineEdit.textChanged.connect(self._generate_address)
        self.passwordChangeButton.clicked.connect(self._generate_wallet_password)
        self.buttonBox.accepted.connect(self.on_accepted_button)
        self.buttonBox.rejected.connect(self.close)

        # fill form
        self._generate_wallet_password()

    def _generate_address(self) -> bool:
        """
        Generate address from mnemonic

        :return:
        """
        self.errorLabel.setText("")
        language_code = self.mnemonic_language_codes[
            self.mnemonicLanguageComboBox.currentIndex()
        ]
        suri = self.mnemonicLineEdit.text().strip()
        if suri == "":
            return False
        try:
            address = Keypair.create_from_uri(
                suri,
                ss58_format=self.application.currencies.get_current().ss58_format,
                crypto_type=KeypairType.SR25519,
                language_code=language_code,
            ).ss58_address
        except Exception as exception:
            logging.exception(exception)
            self.errorLabel.setText(self._("Mnemonic or language not valid!"))
            self.addressValueLabel.setText("")
            self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
            return False

        self.addressValueLabel.setText(address)
        self.buttonBox.button(self.buttonBox.Ok).setEnabled(True)
        return True

    def _generate_wallet_password(self):
        """
        Generate new password for wallet encryption in UI

        :return:
        """
        self.passwordLineEdit.setText(generate_alphabetic(WALLETS_PASSWORD_LENGTH))

    def on_accepted_button(self):
        """
        Triggered when user click on ok button

        :return:
        """
        address = self.addressValueLabel.text()
        suri = self.mnemonicLineEdit.text()
        password = self.passwordLineEdit.text()
        language_code = self.mnemonic_language_codes[
            self.mnemonicLanguageComboBox.currentIndex()
        ]

        # create keypair from mnemonic to get seed as hexadecimal
        keypair = Keypair.create_from_uri(
            suri,
            language_code=language_code,
            crypto_type=KeypairType.SR25519,
            ss58_format=self.application.currencies.get_current().ss58_format,
        )

        wallet = self.application.wallets.get(address)
        if wallet is None:
            # create and store Wallet instance
            wallet = self.application.wallets.create(keypair, password)
            self.application.wallets.add(wallet)
        else:
            # display confirm dialog and get response
            button = QMessageBox.question(
                self,
                self._("Change wallet password?"),
                self._(
                    "A wallet already exists for this account. Change password for {address} wallet?"
                ).format(address=wallet.address),
            )
            if button == QMessageBox.Yes:
                # create and store Wallet instance
                new_wallet = self.application.wallets.create(keypair, password)
                self.application.wallets.update(new_wallet)

        account = self.application.accounts.get_by_address(address)
        if account is None:
            # create and store Account instance
            account = Account(
                address, crypto_type=KeypairType.SR25519, _keypair=keypair
            )
            self.application.accounts.add(account)
        else:
            self.application.accounts.unlock(account, password)
            self.application.accounts.update(account)


if __name__ == "__main__":
    qapp = QApplication(sys.argv)
    application_ = Application(DATA_PATH)
    AccountImportWindow(application_).exec_()



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import sys
from typing import TYPE_CHECKING, Optional

from PyQt5.QtWidgets import QApplication, QDialog, QLineEdit, QWidget

from tikka.domains.application import Application
from tikka.domains.entities.account import Account
from tikka.domains.entities.constants import DATA_PATH, WALLETS_PASSWORD_LENGTH
from tikka.slots.pyqt.resources.gui.windows.account_unlock_rc import Ui_UnlockDialog

if TYPE_CHECKING:
    import _


class AccountUnlockWindow(QDialog, Ui_UnlockDialog):
    """
    AccountUnlockWindow class
    """

    def __init__(
        self,
        application: Application,
        account: Account,
        parent: Optional[QWidget] = None,
    ):
        """
        Init unlock account window

        :param application: Application instance
        :param account: Account instance
        :param parent: QWidget instance
        """
        super().__init__(parent=parent)
        self.setupUi(self)

        self.application = application
        self.account = account
        self._ = self.application.translator.gettext

        # populate fields
        self.addressValueLabel.setText(self.account.address)
        # disable Ok button
        self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)

        # events
        self.showButton.clicked.connect(self.on_show_button_clicked)
        self.passwordLineEdit.textChanged.connect(
            self._on_password_line_edit_text_changed
        )
        self.buttonBox.rejected.connect(self.on_rejected_button)

    def _on_password_line_edit_text_changed(self):
        """
        Triggered when text is changed in the password field

        :return:
        """
        if len(self.passwordLineEdit.text()) == WALLETS_PASSWORD_LENGTH:
            self._unlock_account()

    def on_show_button_clicked(self):
        """
        Triggered when user click on show button

        :return:
        """
        if self.passwordLineEdit.echoMode() == QLineEdit.Password:
            self.passwordLineEdit.setEchoMode(QLineEdit.Normal)
            self.showButton.setText(self._("Hide"))
        else:
            self.passwordLineEdit.setEchoMode(QLineEdit.Password)
            self.showButton.setText(self._("Show"))

    def _unlock_account(self) -> None:
        """
        Validate fields and unlock account to enabled ok button

        :return:
        """
        password = self.passwordLineEdit.text().strip().upper()
        self.errorLabel.setText("")
        try:
            result = self.application.accounts.unlock(self.account, password)
        except Exception:
            self.errorLabel.setStyleSheet("color: red;")
            self.errorLabel.setText(self._("Password is not valid!"))
            self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
            return None

        if not result:
            self.errorLabel.setStyleSheet("color: red;")
            self.errorLabel.setText(self._("Password is not valid!"))
            self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
            return None

        self.errorLabel.setStyleSheet("color: green;")
        self.errorLabel.setText(self._("Account is unlocked"))
        self.buttonBox.button(self.buttonBox.Ok).setEnabled(True)
        self.buttonBox.button(self.buttonBox.Ok).click()
        return None

    def on_rejected_button(self) -> None:
        """
        Triggered when user click on cancel button

        :return:
        """
        self.application.accounts.lock(self.account)


if __name__ == "__main__":
    qapp = QApplication(sys.argv)
    application_ = Application(DATA_PATH)
    account_ = Account("CUYYUnh7N49WZhs5DULkmqw5Zu5fwsRBmE5LLrUFRpgw")
    AccountUnlockWindow(application_, account_).exec_()



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import logging
import sys
from typing import Optional

from duniterpy.key import SigningKey
from PyQt5 import QtCore, QtWidgets
from PyQt5.QtGui import QKeyEvent
from PyQt5.QtWidgets import QApplication, QDialog, QMessageBox, QWidget
from substrateinterface import Keypair, KeypairType

from tikka.domains.application import Application
from tikka.domains.entities.account import Account
from tikka.domains.entities.constants import DATA_PATH, WALLETS_PASSWORD_LENGTH
from tikka.libs.secret import generate_alphabetic
from tikka.slots.pyqt.resources.gui.windows.v1_account_import_rc import (
    Ui_V1AccountImportDialog,
)


class V1AccountImportWindow(QDialog, Ui_V1AccountImportDialog):
    """
    V1AccountImportWindow class
    """

    def __init__(self, application: Application, parent: Optional[QWidget] = None):
        """
        Init import V1 account window

        :param application: Application instance
        :param parent: QWidget instance
        """
        super().__init__(parent=parent)
        self.setupUi(self)

        self.application = application
        self._ = self.application.translator.gettext

        # buttons
        self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)

        # events
        self.secretIDLineEdit.keyPressEvent = (
            self._on_secret_id_line_edit_keypress_event
        )
        self.passwordIDLineEdit.keyPressEvent = (
            self._on_password_id_line_edit_keypress_event
        )
        self.passwordChangeButton.clicked.connect(self._generate_wallet_password)
        self.buttonBox.accepted.connect(self.on_accepted_button)
        self.buttonBox.rejected.connect(self.close)

        # fill form
        self._generate_wallet_password()

    def _on_secret_id_line_edit_keypress_event(self, event: QKeyEvent):
        """
        Triggered when a key is pressed in the secret ID field

        :return:
        """
        if event.key() == QtCore.Qt.Key_Return:
            self._generate_address()
        else:
            QtWidgets.QLineEdit.keyPressEvent(self.secretIDLineEdit, event)
            # if the key is not return, handle normally

    def _on_password_id_line_edit_keypress_event(self, event: QKeyEvent):
        """
        Triggered when a key is pressed in the password ID field

        :return:
        """
        if event.key() == QtCore.Qt.Key_Return:
            self._generate_address()
        else:
            QtWidgets.QLineEdit.keyPressEvent(self.passwordIDLineEdit, event)
            # if the key is not return, handle normally

    def _generate_address(self) -> bool:
        """
        Generate address from ID

        :return:
        """
        self.v1AddressValueLabel.setText("")
        self.addressValueLabel.setText("")
        self.errorLabel.setText("")
        self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)

        secret_id = self.secretIDLineEdit.text().strip()
        password_id = self.passwordIDLineEdit.text().strip()
        if secret_id == "" or password_id == "":
            return False

        signing_key = SigningKey.from_credentials(secret_id, password_id)
        try:
            address = Keypair.create_from_seed(
                seed_hex=signing_key.seed.hex(),
                ss58_format=self.application.currencies.get_current().ss58_format,
                crypto_type=KeypairType.ED25519,
            ).ss58_address
        except Exception as exception:
            logging.exception(exception)
            self.errorLabel.setText(self._("Error generating account wallet!"))
            return False

        self.addressValueLabel.setText(address)
        self.v1AddressValueLabel.setText(signing_key.pubkey)
        self.buttonBox.button(self.buttonBox.Ok).setEnabled(True)
        return True

    def _generate_wallet_password(self):
        """
        Generate new password for wallet encryption in UI

        :return:
        """
        self.passwordLineEdit.setText(generate_alphabetic(WALLETS_PASSWORD_LENGTH))

    def on_accepted_button(self):
        """
        Triggered when user click on ok button

        :return:
        """
        secret_id = self.secretIDLineEdit.text().strip()
        password_id = self.passwordIDLineEdit.text().strip()
        password = self.passwordLineEdit.text()
        signing_key = SigningKey.from_credentials(secret_id, password_id)

        keypair = Keypair.create_from_seed(
            seed_hex=signing_key.seed.hex(),
            ss58_format=self.application.currencies.get_current().ss58_format,
            crypto_type=KeypairType.ED25519,
        )
        address = keypair.ss58_address

        wallet = self.application.wallets.get(address)
        if wallet is None:
            # create and store Wallet instance
            wallet = self.application.wallets.create(keypair, password)
            self.application.wallets.add(wallet)
        else:
            # display confirm dialog and get response
            button = QMessageBox.question(
                self,
                self._("Change wallet password?"),
                self._(
                    "A wallet already exists for this account. Change password for {address} wallet?"
                ).format(address=wallet.address),
            )
            if button == QMessageBox.Yes:
                # create and store Wallet instance
                new_wallet = self.application.wallets.create(keypair, password)
                self.application.wallets.update(new_wallet)

        account = self.application.accounts.get_by_address(address)
        if account is None:
            # create and store Account instance
            account = Account(
                address,
                crypto_type=KeypairType.ED25519,
                _keypair=keypair,
            )
            self.application.accounts.add(account)
        else:
            self.application.accounts.unlock(account, password)
            self.application.accounts.update(account)


if __name__ == "__main__":
    qapp = QApplication(sys.argv)
    application_ = Application(DATA_PATH)
    V1AccountImportWindow(application_).exec_()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/widgets/account.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AccountWidget(object):
    def setupUi(self, AccountWidget):
        AccountWidget.setObjectName("AccountWidget")
        AccountWidget.resize(615, 384)
        AccountWidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(AccountWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.safeImageLabel = QtWidgets.QLabel(AccountWidget)
        self.safeImageLabel.setMaximumSize(QtCore.QSize(100, 100))
        self.safeImageLabel.setBaseSize(QtCore.QSize(200, 0))
        self.safeImageLabel.setText("")
        self.safeImageLabel.setPixmap(QtGui.QPixmap(":/images/safe"))
        self.safeImageLabel.setScaledContents(True)
        self.safeImageLabel.setObjectName("safeImageLabel")
        self.horizontalLayout.addWidget(self.safeImageLabel)
        self.balanceLabel = QtWidgets.QLabel(AccountWidget)
        font = QtGui.QFont()
        font.setPointSize(40)
        self.balanceLabel.setFont(font)
        self.balanceLabel.setObjectName("balanceLabel")
        self.horizontalLayout.addWidget(self.balanceLabel)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.refreshButton = QtWidgets.QPushButton(AccountWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.refreshButton.sizePolicy().hasHeightForWidth())
        self.refreshButton.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/refresh_icon"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.refreshButton.setIcon(icon)
        self.refreshButton.setObjectName("refreshButton")
        self.verticalLayout.addWidget(self.refreshButton)
        self.transferButton = QtWidgets.QPushButton(AccountWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.transferButton.sizePolicy().hasHeightForWidth())
        self.transferButton.setSizePolicy(sizePolicy)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/payment_icon"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.transferButton.setIcon(icon1)
        self.transferButton.setObjectName("transferButton")
        self.verticalLayout.addWidget(self.transferButton)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setContentsMargins(6, -1, 0, -1)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.lockStatusIcon = QtWidgets.QLabel(AccountWidget)
        self.lockStatusIcon.setMaximumSize(QtCore.QSize(16, 16))
        self.lockStatusIcon.setText("")
        self.lockStatusIcon.setPixmap(QtGui.QPixmap(":/images/locked_cartoon_400x400.png"))
        self.lockStatusIcon.setScaledContents(True)
        self.lockStatusIcon.setObjectName("lockStatusIcon")
        self.horizontalLayout_2.addWidget(self.lockStatusIcon)
        self.addressLabel = QtWidgets.QLabel(AccountWidget)
        self.addressLabel.setText("")
        self.addressLabel.setObjectName("addressLabel")
        self.horizontalLayout_2.addWidget(self.addressLabel)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem1)

        self.retranslateUi(AccountWidget)
        QtCore.QMetaObject.connectSlotsByName(AccountWidget)

    def retranslateUi(self, AccountWidget):
        #  _translate = QtCore.QCoreApplication.translate
        AccountWidget.setWindowTitle(_("Account"))
        self.balanceLabel.setText(_("0"))
        self.refreshButton.setText(_("Refresh"))
        self.transferButton.setText(_("Transfer"))
import tikka.slots.pyqt.resources.icons.index_rc



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import logging
import sys
from typing import Optional

from PyQt5 import QtCore, QtWidgets
from PyQt5.QtGui import QKeyEvent, QMovie
from PyQt5.QtWidgets import QApplication, QDialog, QWidget
from substrateinterface import Keypair, KeypairType

from tikka.domains.application import Application
from tikka.domains.entities.account import Account
from tikka.domains.entities.constants import DATA_PATH
from tikka.interfaces.adapters.repository.preferences import SELECTED_UNIT
from tikka.slots.pyqt.entities.constants import ICON_LOADER
from tikka.slots.pyqt.entities.worker import AsyncQWorker
from tikka.slots.pyqt.resources.gui.windows.transfer_rc import Ui_TransferDialog


class TransferWindow(QDialog, Ui_TransferDialog):
    """
    TransferWindow class
    """

    def __init__(
        self,
        application: Application,
        account: Account,
        parent: Optional[QWidget] = None,
    ):
        """
        Init transfer window

        :param application: Application instance
        :param account: Account instance
        :param parent: QWidget instance
        """
        super().__init__(parent=parent)
        self.setupUi(self)

        # only open if account is unlocked
        if account.keypair is None:
            raise ValueError("TransferWindow: sender account keypair is None")

        self.application = application
        self.account = account
        self._ = self.application.translator.gettext

        # substrate error message for gettext extraction
        self._("Transfer/payment would kill account")
        self._("Balance too low to send value")

        self.unit = self.application.preferences_repository.get(SELECTED_UNIT)
        self.init_units()

        self.recipient_account: Optional[Account] = None
        self.amount_value = 0
        self.fees = None
        self.transfer_success: Optional[bool] = None

        self.senderAddressValueLabel.setText(account.address)
        self.feesButton.setDisabled(True)
        self.sendButton.setDisabled(True)

        # animated loading icon
        self.loader_movie = QMovie(ICON_LOADER)
        self.loader_movie.start()
        self.loaderIconLabel.setMovie(self.loader_movie)
        self.loaderIconLabel.hide()

        self._update_ui()

        # events
        self.recipientAddressLineEdit.keyPressEvent = (
            self._on_recipient_address_line_edit_keypress_event
        )
        self.amountDoubleSpinBox.valueChanged.connect(
            self._on_amount_double_spin_box_changed
        )
        self.amountUnitComboBox.activated.connect(self._on_unit_changed)
        self.feesButton.clicked.connect(self._on_fees_button_clicked)
        self.sendButton.clicked.connect(self._on_send_button_clicked)
        self.buttonBox.button(self.buttonBox.Close).clicked.connect(self.close)

        ##############################
        # ASYNC METHODS
        ##############################
        # fetch recipient balance
        self.fetch_recipient_balance_from_network_async_qworker = AsyncQWorker(
            self.fetch_recipient_balance_from_network
        )
        self.fetch_recipient_balance_from_network_async_qworker.finished.connect(
            self._on_finished_fetch_recipient_balance_from_network
        )
        # fetch fees from network
        self.fetch_fees_from_network_async_qworker = AsyncQWorker(
            self.fetch_fees_from_network
        )
        self.fetch_fees_from_network_async_qworker.finished.connect(
            self._on_finished_fetch_fees_from_network
        )
        # send transfer to network
        self.send_tranfer_to_network_async_qworker = AsyncQWorker(
            self.send_transfer_to_network
        )
        self.send_tranfer_to_network_async_qworker.finished.connect(
            self._on_finished_send_transfer_to_network
        )

    def _on_unit_changed(self):
        """
        Triggered when unit combo box is changed

        :return:
        """
        self.unit = self.application.amounts.get_register_keys()[
            self.amountUnitComboBox.currentIndex()
        ]
        self._update_ui()

    def _on_amount_double_spin_box_changed(self):
        """
        Triggered when the amount spin box is changed

        :return:
        """
        self.amount_value = self.amountDoubleSpinBox.value()
        if self.recipient_account is not None and self.amount_value > 0:
            self.feesButton.setDisabled(False)
            self.sendButton.setDisabled(False)

    def fetch_recipient_balance_from_network(self):
        """
        Fetch last account data from the network

        :return:
        """
        self.loaderIconLabel.show()
        self.application.accounts.fetch_balance_from_network(self.recipient_account)

    def _on_finished_fetch_recipient_balance_from_network(self):
        """
        Triggered when async request fetch_from_network is finished

        :return:
        """
        self.loaderIconLabel.hide()
        self._update_ui()

    def _on_recipient_address_line_edit_keypress_event(self, event: QKeyEvent):
        """
        Triggered when a key is pressed in the recipient address field

        :return:
        """
        if event.key() == QtCore.Qt.Key_Return:
            self._get_recipient_account()
            if self.recipient_account is not None and self.amount_value > 0:
                self.feesButton.setDisabled(False)
                self.sendButton.setDisabled(False)
        else:
            QtWidgets.QLineEdit.keyPressEvent(self.recipientAddressLineEdit, event)
            # if the key is not return, handle normally

    def _get_recipient_account(self) -> None:
        """
        Get recipient account

        :return:
        """
        self.recipient_account = None
        address = self.recipientAddressLineEdit.text().strip()
        try:
            Keypair(
                ss58_address=address,
                ss58_format=self.application.currencies.get_current().ss58_format,
            )
        except ValueError as exception:
            logging.exception(exception)
            self.recipientBalanceValueLabel.setText(self._("Invalid!"))
            self.recipientBalanceValueLabel.setToolTip(
                self._("Invalid address! Please check it again.")
            )
            self.recipient_account = None
            return None

        # search in local account list
        for account in self.application.accounts.list:
            if account.address == address:
                self.recipient_account = account
                self._update_ui()
                return None

        # create account and fetch balance
        self.recipient_account = Account(address)
        self.fetch_recipient_balance_from_network_async_qworker.start()

        return None

    def _update_ui(self):
        """
        Update UI

        :return:
        """
        amount = self.application.amounts.get_amount(self.unit)
        self.senderBalanceValueLabel.setText(
            self.locale().toCurrencyString(
                amount.value(self.account.balance), amount.symbol()
            )
        )
        if self.fees is not None:
            self.feesValueLabel.setText(
                self.locale().toCurrencyString(amount.value(self.fees), amount.symbol())
            )
        else:
            self.feesValueLabel.setText("")

        if self.recipientAddressLineEdit.text().strip() != "":
            if self.recipient_account.balance is None:
                self.recipientBalanceValueLabel.setText(self._("Unknown!"))
                self.recipientBalanceValueLabel.setToolTip(
                    self._(
                        "Account balance unknown! Send only one unit and make sure the owner can get it"
                    )
                )
            else:
                self.recipientBalanceValueLabel.setText(
                    self.locale().toCurrencyString(
                        amount.value(self.recipient_account.balance), amount.symbol()
                    )
                )
                self.recipientBalanceValueLabel.setToolTip("")

    def init_units(self) -> None:
        """
        Init units combobox for transfer amount

        :return:
        """
        self.amountUnitComboBox.clear()

        self.amountUnitComboBox.addItems(self.application.amounts.get_register_names())
        self.amountUnitComboBox.setCurrentIndex(
            self.application.amounts.get_register_keys().index(self.unit)
        )

    def _on_fees_button_clicked(self):
        """
        Triggered when user click on Fees button

        :return:
        """
        self.feesButton.setDisabled(True)
        self.loaderIconLabel.show()

        self.fetch_fees_from_network_async_qworker.start()

    def fetch_fees_from_network(self):
        """
        Fetch fees amount from network

        :return:
        """
        # get value as blockchain units
        amount = self.application.amounts.get_amount(self.unit)
        blockchain_value = amount.blockchain_value(self.amountDoubleSpinBox.value())
        self.fees = self.application.transfers.fees(
            self.account, self.recipient_account.address, blockchain_value
        )

    def _on_finished_fetch_fees_from_network(self):
        """
        Triggered when async request fetch_from_network is finished

        :return:
        """
        self.loaderIconLabel.hide()
        self.feesButton.setDisabled(False)
        self._update_ui()

    def _on_send_button_clicked(self):
        """
        Triggered when user click on Send button

        :return:
        """
        self.sendButton.setDisabled(True)
        self.loaderIconLabel.show()

        self.send_tranfer_to_network_async_qworker.start()

    def send_transfer_to_network(self):
        """
        Send transfer to network

        :return:
        """
        self.sendStatusLabel.setText("")
        # get value as blockchain units
        amount = self.application.amounts.get_amount(self.unit)
        blockchain_value = amount.blockchain_value(self.amountDoubleSpinBox.value())
        extrinsic_receipt = self.application.transfers.send(
            self.account, self.recipient_account.address, blockchain_value
        )
        if extrinsic_receipt is None:
            self.transfer_success = False
            self.sendStatusLabel.setStyleSheet("color: red;")
            self.sendStatusLabel.setText(
                self._("Transfer failed. Please check logs to understand why")
            )
        elif extrinsic_receipt.is_success is False:
            self.transfer_success = False
            self.sendStatusLabel.setStyleSheet("color: red;")
            self.sendStatusLabel.setText(
                self._(extrinsic_receipt.error_message["docs"][0])
            )
        else:
            self.transfer_success = True
            self.sendStatusLabel.setStyleSheet("color: green;")
            self.sendStatusLabel.setText(self._("Transfer done"))

    def _on_finished_send_transfer_to_network(self):
        """
        Triggered when async request send_transfer_to_network is finished

        :return:
        """
        self.loaderIconLabel.hide()
        self.sendButton.setDisabled(False)


if __name__ == "__main__":
    qapp = QApplication(sys.argv)
    application_ = Application(DATA_PATH)
    account_ = Account("5E6taVoL7eW8qpPKjDcf3GjyvFkkMB8EHkkqZksiWeViEkSP", balance=0)
    account_.keypair = Keypair.create_from_seed(
        seed_hex="4c751d1d630ef33e1c3e0e6926a928a5ce229baacedaaccc1c2de15a5443fb1b",
        crypto_type=KeypairType.SR25519,
        ss58_format=application_.currencies.get_current().ss58_format,
    )

    TransferWindow(application_, account_).exec_()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/widgets/account_list.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AccountListWidget(object):
    def setupUi(self, AccountListWidget):
        AccountListWidget.setObjectName("AccountListWidget")
        AccountListWidget.resize(565, 296)
        self.verticalLayout = QtWidgets.QVBoxLayout(AccountListWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(AccountListWidget)
        font = QtGui.QFont()
        font.setFamily("FreeSans")
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label, 0, QtCore.Qt.AlignHCenter)
        self.tableWidget = QtWidgets.QTableWidget(AccountListWidget)
        self.tableWidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tableWidget.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.tableWidget.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.tableWidget.setAutoScroll(True)
        self.tableWidget.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tableWidget.setObjectName("tableWidget")
        self.tableWidget.setColumnCount(2)
        self.tableWidget.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        item.setTextAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignVCenter)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        item.setFont(font)
        self.tableWidget.setHorizontalHeaderItem(1, item)
        self.tableWidget.horizontalHeader().setDefaultSectionSize(50)
        self.tableWidget.horizontalHeader().setMinimumSectionSize(50)
        self.tableWidget.horizontalHeader().setStretchLastSection(True)
        self.tableWidget.verticalHeader().setVisible(False)
        self.verticalLayout.addWidget(self.tableWidget)

        self.retranslateUi(AccountListWidget)
        QtCore.QMetaObject.connectSlotsByName(AccountListWidget)

    def retranslateUi(self, AccountListWidget):
        #  _translate = QtCore.QCoreApplication.translate
        AccountListWidget.setWindowTitle(_("Account list"))
        self.label.setText(_("Account list"))
        item = self.tableWidget.horizontalHeaderItem(1)
        item.setText(_("Address"))
import tikka.slots.pyqt.resources.icons.index_rc



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from typing import List, Optional
from uuid import UUID

from tikka.domains.entities.events import NodesEvent
from tikka.domains.entities.node import Node
from tikka.domains.events import EventDispatcher
from tikka.interfaces.adapters.network.nodes import NetworkNodesInterface
from tikka.interfaces.adapters.repository.nodes import NodesRepositoryInterface


class Nodes:
    """
    Nodes domain class
    """

    DEFAULT_LIST_OFFSET = 0
    DEFAULT_LIST_LIMIT = 1000

    SORT_ORDER_ASCENDING = NodesRepositoryInterface.SORT_ORDER_ASCENDING
    SORT_ORDER_DESCENDING = NodesRepositoryInterface.SORT_ORDER_DESCENDING

    PROPERTY_ID = NodesRepositoryInterface.COLUMN_ID
    PROPERTY_PEER_ID = NodesRepositoryInterface.COLUMN_PEER_ID
    PROPERTY_BLOCK = NodesRepositoryInterface.COLUMN_BLOCK
    PROPERTY_SOFTWARE = NodesRepositoryInterface.COLUMN_SOFTWARE
    PROPERTY_SOFTWARE_VERSION = NodesRepositoryInterface.COLUMN_SOFTWARE_VERSION

    def __init__(
        self,
        repository: NodesRepositoryInterface,
        network: NetworkNodesInterface,
        event_dispatcher: EventDispatcher,
    ):
        """
        Init Nodes domain

        :param repository: Database adapter instance
        :param network: Network adapter instance for handling nodes
        :param event_dispatcher: EventDispatcher instance
        """
        self.repository = repository
        self.network = network
        self.event_dispatcher = event_dispatcher

    def add(self, node: Node):
        """
        Add node

        :param node: Node instance
        :return:
        """
        # add account
        self.repository.add(node)

        self.event_dispatcher.dispatch_event(
            NodesEvent(NodesEvent.EVENT_TYPE_LIST_CHANGED)
        )

    def update(self, node: Node):
        """
        Update node

        :param node: Node instance
        :return:
        """
        # update only non hidden fields
        self.repository.update(node)

        self.event_dispatcher.dispatch_event(
            NodesEvent(NodesEvent.EVENT_TYPE_LIST_CHANGED)
        )

    def get(self, id: UUID) -> Optional[Node]:  # pylint: disable=redefined-builtin
        """
        Return Node instance from ID

        :param id: Node ID
        :return:
        """
        return self.repository.get(id)

    def list(
        self,
        offset: int = DEFAULT_LIST_OFFSET,
        limit: int = DEFAULT_LIST_LIMIT,
        sort_order: int = SORT_ORDER_ASCENDING,
        sort_property: str = PROPERTY_ID,
    ) -> List[Node]:
        """
        List nodes from repository

        :param offset: Offset index to get rows from
        :param limit: Number of rows to return
        :param sort_order: Sort order flag
        :param sort_property: Property flag to sort by
        :return:
        """
        return self.repository.list(
            offset,
            limit,
            sort_order,
            sort_property,
        )

    def delete(self, id: UUID) -> None:  # pylint: disable=redefined-builtin
        """
        Delete node by ID

        :param id: Node ID
        :return:
        """
        self.repository.delete(id)

        self.event_dispatcher.dispatch_event(
            NodesEvent(NodesEvent.EVENT_TYPE_LIST_CHANGED)
        )

    def count(self) -> int:
        """
        Return total node count

        :return:
        """
        return self.repository.count()

    def fetch_from_network(self) -> Optional[Node]:
        """
        Fetch Node from current EntryPoint connection

        :return:
        """
        return self.network.get()



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import collections
from gettext import NullTranslations
from typing import Dict

from tikka.domains.currencies import Currencies
from tikka.domains.entities.amounts import UnitAmount, UniversalDividendAmount
from tikka.domains.entities.constants import AMOUNT_UNIT_KEY
from tikka.interfaces.entities.amount import AmountInterface


class Amounts:
    """
    Amounts domain class
    """

    def __init__(self, currencies: Currencies, translator: NullTranslations):
        """
        Init Amounts domain instance

        :param currencies: Currencies domain instance
        :param translator: Gnu gettext NullTranslations instance
        """
        self.currencies = currencies
        self.translator = translator
        self.register: Dict[str, AmountInterface] = collections.OrderedDict()

        self.add_amount(AMOUNT_UNIT_KEY, UnitAmount(self.currencies, self.translator))
        self.add_amount("ud", UniversalDividendAmount(self.currencies, self.translator))

    def add_amount(self, name: str, amount: AmountInterface) -> None:
        """
        Add AmountInterface instance in the register with referential name as key

        :param name: Name of amount referential
        :param amount: AmountInterface instance
        :return:
        """
        self.register[name] = amount

    def get_amount(
        self,
        type_: str,
    ) -> AmountInterface:
        """
        Get AmountInterface instance  from register by type name

        :param type_: Name of amount type
        :param blockchain_value: Value from blockchain
        :return:
        """
        return self.register[type_]

    def get_register_keys(self) -> list:
        """
        Return register keys as list

        :return:
        """
        return list(self.register.keys())

    def get_register_names(self) -> list:
        """
        Return register unit names as list

        :return:
        """
        return [amount.name() for amount in self.register.values()]



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/account_unlock.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_UnlockDialog(object):
    def setupUi(self, UnlockDialog):
        UnlockDialog.setObjectName("UnlockDialog")
        UnlockDialog.resize(781, 334)
        UnlockDialog.setStyleSheet("")
        self.verticalLayout = QtWidgets.QVBoxLayout(UnlockDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.imageLabel = QtWidgets.QLabel(UnlockDialog)
        self.imageLabel.setMaximumSize(QtCore.QSize(200, 200))
        self.imageLabel.setText("")
        self.imageLabel.setPixmap(QtGui.QPixmap(":/images/unlocked"))
        self.imageLabel.setScaledContents(True)
        self.imageLabel.setObjectName("imageLabel")
        self.horizontalLayout.addWidget(self.imageLabel)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.addressLabel = QtWidgets.QLabel(UnlockDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.addressLabel.setFont(font)
        self.addressLabel.setObjectName("addressLabel")
        self.gridLayout.addWidget(self.addressLabel, 0, 0, 1, 1)
        self.passwordLabel = QtWidgets.QLabel(UnlockDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.passwordLabel.sizePolicy().hasHeightForWidth())
        self.passwordLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.passwordLabel.setFont(font)
        self.passwordLabel.setObjectName("passwordLabel")
        self.gridLayout.addWidget(self.passwordLabel, 1, 0, 1, 1)
        self.passwordLineEdit = QtWidgets.QLineEdit(UnlockDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.passwordLineEdit.sizePolicy().hasHeightForWidth())
        self.passwordLineEdit.setSizePolicy(sizePolicy)
        self.passwordLineEdit.setText("")
        self.passwordLineEdit.setEchoMode(QtWidgets.QLineEdit.Password)
        self.passwordLineEdit.setAlignment(QtCore.Qt.AlignCenter)
        self.passwordLineEdit.setObjectName("passwordLineEdit")
        self.gridLayout.addWidget(self.passwordLineEdit, 1, 1, 1, 1)
        self.showButton = QtWidgets.QPushButton(UnlockDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.showButton.sizePolicy().hasHeightForWidth())
        self.showButton.setSizePolicy(sizePolicy)
        self.showButton.setObjectName("showButton")
        self.gridLayout.addWidget(self.showButton, 1, 2, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 1, 4, 1, 1)
        self.addressValueLabel = QtWidgets.QLabel(UnlockDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.addressValueLabel.sizePolicy().hasHeightForWidth())
        self.addressValueLabel.setSizePolicy(sizePolicy)
        self.addressValueLabel.setText("")
        self.addressValueLabel.setObjectName("addressValueLabel")
        self.gridLayout.addWidget(self.addressValueLabel, 0, 1, 1, 4)
        self.verticalLayout.addLayout(self.gridLayout)
        self.errorLabel = QtWidgets.QLabel(UnlockDialog)
        self.errorLabel.setStyleSheet("color : rgb(255, 0, 0)")
        self.errorLabel.setText("")
        self.errorLabel.setObjectName("errorLabel")
        self.verticalLayout.addWidget(self.errorLabel, 0, QtCore.Qt.AlignHCenter)
        self.buttonBox = QtWidgets.QDialogButtonBox(UnlockDialog)
        self.buttonBox.setEnabled(True)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox, 0, QtCore.Qt.AlignHCenter)

        self.retranslateUi(UnlockDialog)
        self.buttonBox.accepted.connect(UnlockDialog.accept) # type: ignore
        self.buttonBox.rejected.connect(UnlockDialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(UnlockDialog)

    def retranslateUi(self, UnlockDialog):
        #  _translate = QtCore.QCoreApplication.translate
        UnlockDialog.setWindowTitle(_("Unlock account"))
        self.addressLabel.setText(_("Address"))
        self.passwordLabel.setText(_("Password"))
        self.showButton.setText(_("Show"))
import tikka.slots.pyqt.resources.icons.index_rc



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
from typing import Optional

from PyQt5.QtWidgets import QApplication, QDialog, QRadioButton, QWidget

from tikka.domains.application import Application
from tikka.domains.config import Config
from tikka.domains.entities.constants import DATA_PATH, LANGUAGES
from tikka.slots.pyqt.resources.gui.windows.configuration_rc import (
    Ui_ConfigurationDialog,
)


class ConfigurationWindow(QDialog, Ui_ConfigurationDialog):
    """
    ConfigurationWindow class
    """

    def __init__(self, application: Application, parent: Optional[QWidget] = None):
        """
        Init configuration window

        :param application: Application instance
        :param parent: QWidget instance
        """
        super().__init__(parent=parent)
        self.setupUi(self)

        self.application = application
        self._ = self.application.translator.gettext

        # language
        self.language_radio_buttons = {}
        for language_code_name in LANGUAGES:
            self.language_radio_buttons[language_code_name] = QRadioButton()
            if language_code_name == self.application.config.get(Config.LANGUAGE_KEY):
                self.language_radio_buttons[language_code_name].setChecked(True)
            self.language_radio_buttons[language_code_name].setText(
                LANGUAGES[language_code_name]
            )
            self.language_radio_buttons[language_code_name].setObjectName(
                f"languageRadioButton_{language_code_name}"
            )
            self.languageRadioButtonsLayout.addWidget(
                self.language_radio_buttons[language_code_name]
            )

        # currencies
        currency_name_by_code_name = dict(
            zip(
                self.application.currencies.code_names(),
                self.application.currencies.names(),
            )
        )
        self.currency_radio_buttons = {}
        for currency_code_name in currency_name_by_code_name:
            self.currency_radio_buttons[currency_code_name] = QRadioButton()
            if currency_code_name == self.application.config.get(Config.CURRENCY_KEY):
                self.currency_radio_buttons[currency_code_name].setChecked(True)
            self.currency_radio_buttons[currency_code_name].setText(
                self._(currency_name_by_code_name[currency_code_name])
            )
            self.currency_radio_buttons[currency_code_name].setObjectName(
                f"currency_radio_button_{currency_code_name}"
            )
            self.currencyRadioButtonsLayout.addWidget(
                self.currency_radio_buttons[currency_code_name]
            )

        # events
        self.buttonBox.button(self.buttonBox.Save).clicked.connect(self.save)
        self.buttonBox.button(self.buttonBox.Cancel).clicked.connect(self.close)

    def save(self):
        """
        Save configuration

        :return:
        """
        # language
        for language_code_name, radio_button in self.language_radio_buttons.items():
            if (
                radio_button.isChecked()
                and language_code_name
                != self.application.config.get(Config.LANGUAGE_KEY)
            ):
                self.application.select_language(language_code_name)

        for currency_code_name, radio_button in self.currency_radio_buttons.items():
            if (
                radio_button.isChecked()
                and currency_code_name
                != self.application.config.get(Config.CURRENCY_KEY)
            ):
                self.application.select_currency(currency_code_name)

        self.close()


if __name__ == "__main__":
    qapp = QApplication(sys.argv)
    application_ = Application(DATA_PATH)
    ConfigurationWindow(application_).exec_()



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import abc
from typing import List, Optional
from uuid import UUID

from tikka.domains.entities.entry_point import EntryPoint


class EntryPointsRepositoryInterface(abc.ABC):
    """
    EntryPointsRepositoryInterface class
    """

    SORT_ORDER_ASCENDING = 0
    SORT_ORDER_DESCENDING = 1

    COLUMN_URL = "url"
    COLUMN_NODE_ID = "node_id"

    DEFAULT_LIST_OFFSET = 0
    DEFAULT_LIST_LIMIT = 1000

    @abc.abstractmethod
    def add(self, entry_point: EntryPoint) -> None:
        """
        Add a new entry point in repository

        :param entry_point: EntryPoint instance
        :return:
        """
        raise NotImplementedError

    @abc.abstractmethod
    def get(self, url: str) -> Optional[EntryPoint]:
        """
        Get entry point by url from repository

        :param url: EntryPoint instance
        :return:
        """
        raise NotImplementedError

    @abc.abstractmethod
    def update(self, entry_point: EntryPoint) -> None:
        """
        Update entry point in repository

        :param entry_point: EntryPoint instance
        :return:
        """
        raise NotImplementedError

    @abc.abstractmethod
    def list(
        self,
        offset: int = DEFAULT_LIST_OFFSET,
        limit: int = DEFAULT_LIST_LIMIT,
        sort_order: int = SORT_ORDER_ASCENDING,
        sort_column: str = COLUMN_URL,
        node_id: Optional[UUID] = None,
    ) -> List[EntryPoint]:
        """
        List entry points from repository

        :param offset: Offset index to get rows from
        :param limit: Number of rows to return
        :param sort_order: Sort order
        :param sort_column: Sort column name
        :param node_id: Filter by node ID
        :return:
        """
        raise NotImplementedError

    @abc.abstractmethod
    def delete(self, url: str) -> None:
        """
        Delete entry point in repository

        :param url: Url of EntryPoint to delete
        :return:
        """
        raise NotImplementedError



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from typing import Any, List, Optional
from uuid import UUID

from tikka.adapters.repository.sqlite3 import Sqlite3RepositoryInterface
from tikka.domains.entities.entry_point import EntryPoint
from tikka.interfaces.adapters.repository.entry_points import (
    EntryPointsRepositoryInterface,
)

TABLE_NAME = "entry_points"

SQL_COLUMNS = {
    EntryPointsRepositoryInterface.COLUMN_URL: "url",
    EntryPointsRepositoryInterface.COLUMN_NODE_ID: "node_id",
}
SQL_SORT_ORDER = {
    EntryPointsRepositoryInterface.SORT_ORDER_ASCENDING: "ASC",
    EntryPointsRepositoryInterface.SORT_ORDER_DESCENDING: "DESC",
}

DEFAULT_LIST_OFFSET = 0
DEFAULT_LIST_LIMIT = 1000


class Sqlite3EntryPointsRepository(
    EntryPointsRepositoryInterface, Sqlite3RepositoryInterface
):
    """
    Sqlite3EntryPointsRepository class
    """

    def list(
        self,
        offset: int = DEFAULT_LIST_OFFSET,
        limit: int = DEFAULT_LIST_LIMIT,
        sort_order: int = EntryPointsRepositoryInterface.SORT_ORDER_ASCENDING,
        sort_column: str = EntryPointsRepositoryInterface.COLUMN_URL,
        node_id: Optional[UUID] = None,
    ) -> List[EntryPoint]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            EntryPointsRepositoryInterface.list.__doc__
        )
        # map interface flags to SQL
        sql_sort_column = SQL_COLUMNS[sort_column]
        sql_sort_order = SQL_SORT_ORDER[sort_order]

        # filter by node_id
        where = ""
        sql_args: List[Any] = []
        if node_id is not None:
            where += f"{SQL_COLUMNS[EntryPointsRepositoryInterface.COLUMN_NODE_ID]} = ?"
            sql_args.append(node_id.hex)
        if where != "":
            where = f"WHERE {where}"

        sql = f"SELECT * FROM {TABLE_NAME} {where} ORDER BY {sql_sort_column} {sql_sort_order} LIMIT {limit} OFFSET {offset}"
        if len(sql_args) > 0:
            result_set = self.client.select(sql, sql_args)
        else:
            result_set = self.client.select(sql)

        list_ = []
        for row in result_set:
            list_.append(get_entry_point_from_row(row))

        return list_

    def add(self, entry_point: EntryPoint) -> None:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            EntryPointsRepositoryInterface.add.__doc__
        )

        # insert only non hidden fields
        self.client.insert(
            TABLE_NAME,
            **get_fields_from_entry_point(entry_point),
        )

    def get(self, url: str) -> Optional[EntryPoint]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            EntryPointsRepositoryInterface.list.__doc__
        )

        row = self.client.select_one(f"SELECT * FROM {TABLE_NAME} WHERE url=?", (url,))

        if row is None:
            return None

        return get_entry_point_from_row(row)

    def update(self, entry_point: EntryPoint) -> None:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            EntryPointsRepositoryInterface.update.__doc__
        )
        # update only non hidden fields
        self.client.update(
            TABLE_NAME,
            f"url='{entry_point.url}'",
            **get_fields_from_entry_point(entry_point),
        )

    def delete(self, url: str) -> None:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            EntryPointsRepositoryInterface.delete.__doc__
        )

        self.client.delete(TABLE_NAME, url=url)


def get_fields_from_entry_point(entry_point: EntryPoint) -> dict:
    """
    Return a dict of supported fields with normalized value

    :param entry_point: EntryPoint instance
    :return:
    """
    fields = {}
    for (key, value) in entry_point.__dict__.items():
        if key.startswith("_"):
            continue
        if key == "node_id" and value is not None:
            # node id UUID to hex str
            value = value.hex
        fields[key] = value

    return fields


def get_entry_point_from_row(row: tuple) -> EntryPoint:
    """
    Return a EntryPoint instance from a result set row

    :param row: Result set row
    :return:
    """
    values = list(row)
    # node id str to UUID
    if values[1] is not None:
        values[1] = UUID(row[1])

    return EntryPoint(*values)



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import sys
from typing import Optional

from PyQt5.QtWidgets import QApplication, QMenu, QMessageBox, QWidget
from substrateinterface import KeypairType

from tikka.domains.application import Application
from tikka.domains.entities.account import Account
from tikka.domains.entities.constants import DATA_PATH
from tikka.slots.pyqt.windows.account_import import AccountImportWindow
from tikka.slots.pyqt.windows.account_unlock import AccountUnlockWindow
from tikka.slots.pyqt.windows.transfer import TransferWindow
from tikka.slots.pyqt.windows.v1_account_import import V1AccountImportWindow
from tikka.slots.pyqt.windows.wallet_password_change import WalletPasswordChangeWindow


class AccountPopupMenu(QMenu):
    """
    AccountPopupMenu class
    """

    def __init__(
        self,
        application: Application,
        account: Account,
        parent: Optional[QWidget] = None,
    ):
        """
        Init AccountPopupMenu instance

        :param application: Application instance
        :param account: Account instance
        :param parent: QWidget instance
        """
        super().__init__(parent=parent)

        self.application = application
        self._ = self.application.translator.gettext

        self.account = account

        # menu actions
        copy_address_to_clipboard_action = self.addAction(
            self._("Copy address to clipboard")
        )
        copy_address_to_clipboard_action.triggered.connect(
            self.copy_address_to_clipboard
        )
        if self.application.wallets.exists(account.address):
            if account.keypair is None:
                unlock_account_action = self.addAction(self._("Unlock account access"))
                unlock_account_action.triggered.connect(self.unlock_account)
            else:
                transfer_action = self.addAction(self._("Make a transfer"))
                transfer_action.triggered.connect(self.transfer)

                lock_account_action = self.addAction(self._("Lock account access"))
                lock_account_action.triggered.connect(self.lock_account)

            wallet_change_password_action = self.addAction(
                self._("Change wallet password")
            )
            wallet_change_password_action.triggered.connect(self.change_wallet_password)
            wallet_password_forgotten_action = self.addAction(
                self._("Wallet password forgotten")
            )
            wallet_password_forgotten_action.triggered.connect(
                self.wallet_password_forgotten
            )

        forget_account_action = self.addAction(self._("Forget account"))
        forget_account_action.triggered.connect(self.confirm_forget_account)

    def copy_address_to_clipboard(self):
        """
        Copy address of selected row to clipboard

        :return:
        """
        clipboard = QApplication.clipboard()
        clipboard.setText(self.account.address)

    def unlock_account(self):
        """
        Open account unlock window

        :return:
        """
        AccountUnlockWindow(self.application, self.account, self).exec_()

    def transfer(self):
        """
        Open transfer window

        :return:
        """
        TransferWindow(self.application, self.account, self).exec_()

    def lock_account(self):
        """
        Lock account

        :return:
        """
        self.application.accounts.lock(self.account)

    def change_wallet_password(self):
        """
        Open change wallet password window

        :return:
        """
        WalletPasswordChangeWindow(self.application, self.account, self).exec_()

    def wallet_password_forgotten(self):
        """
        Open add account with credentials to reset wallet password

        :return:
        """
        # if wallet type is V2...
        if self.account.crypto_type == KeypairType.SR25519:
            AccountImportWindow(self.application).exec_()
        # wallet V1
        else:
            V1AccountImportWindow(self.application).exec_()

    def confirm_forget_account(self):
        """
        Display confirm dialog then forget account if confirmed

        :return:
        """
        # display confirm dialog and get response
        button = QMessageBox.question(
            self,
            self._("Forget account"),
            self._("Forget account {address}?").format(address=self.account.address),
        )
        if button == QMessageBox.Yes:
            self.application.accounts.delete(self.account)


if __name__ == "__main__":
    qapp = QApplication(sys.argv)

    application_ = Application(DATA_PATH)
    account_ = Account("732SSfuwjB7jkt9th1zerGhphs6nknaCBCTozxUcPWPU")

    menu = AccountPopupMenu(application_, account_)
    menu.exec_()

    sys.exit(qapp.exec_())



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from typing import List, Optional
from uuid import UUID

from tikka.domains.config import Config
from tikka.domains.connections import Connections
from tikka.domains.currencies import Currencies
from tikka.domains.entities.entry_point import EntryPoint
from tikka.domains.entities.events import ConnectionsEvent
from tikka.domains.events import EventDispatcher
from tikka.domains.nodes import Nodes
from tikka.interfaces.adapters.repository.entry_points import (
    EntryPointsRepositoryInterface,
)
from tikka.interfaces.adapters.repository.preferences import (
    CURRENT_ENTRY_POINT_URL,
    PreferencesRepositoryInterface,
)


class EntryPoints:
    """
    EntryPoints domain class
    """

    DEFAULT_LIST_OFFSET = 0
    DEFAULT_LIST_LIMIT = 1000

    SORT_ORDER_ASCENDING = EntryPointsRepositoryInterface.SORT_ORDER_ASCENDING
    SORT_ORDER_DESCENDING = EntryPointsRepositoryInterface.SORT_ORDER_DESCENDING

    PROPERTY_URL = EntryPointsRepositoryInterface.COLUMN_URL
    PROPERTY_NODE_ID = EntryPointsRepositoryInterface.COLUMN_NODE_ID

    def __init__(
        self,
        repository: EntryPointsRepositoryInterface,
        preferences_repository: PreferencesRepositoryInterface,
        connections: Connections,
        nodes: Nodes,
        config: Config,
        currencies: Currencies,
        event_dispatcher: EventDispatcher,
    ):
        """
        Init EntryPoints domain instance

        :param repository: EntryPointsRepositoryInterface instance
        :param preferences_repository: PreferencesRepositoryInterface instance
        :param connections: Connections instance
        :param nodes: Nodes instance
        :param config: Config instance
        :param currencies: Currencies instance
        :param event_dispatcher: EventDispatcher instance
        """
        self.repository = repository
        self.preferences_repository = preferences_repository
        self.connections = connections
        self.nodes = nodes
        self.config = config
        self.currencies = currencies
        self.event_dispatcher = event_dispatcher
        self.current_url = self.currencies.get_entry_point_urls()[0]

        self.init_repository()

        # events
        self.event_dispatcher.add_event_listener(
            ConnectionsEvent.EVENT_TYPE_CONNECTED, self._on_connections_connected
        )

    def init_repository(self):
        """
        Init repository with default entry points from config

        :return:
        """
        # init repository with config default entry points
        if len(self.repository.list()) == 0:
            for url in self.currencies.get_entry_point_urls():
                self.repository.add(EntryPoint(url))

        self.current_url = self.repository.list(0, 1)[0].url

        current_url_in_preferences = self.preferences_repository.get(
            CURRENT_ENTRY_POINT_URL
        )
        if current_url_in_preferences is None:
            self.preferences_repository.set(CURRENT_ENTRY_POINT_URL, self.current_url)
        else:
            self.current_url = current_url_in_preferences

    def network_fetch_current_node(self) -> None:
        """
        Add or update node from current entry point

        :return:
        """
        node = self.nodes.fetch_from_network()
        if node is None:
            return None
        entry_point = self.get(self.get_current_url())
        if entry_point is None:
            return None
        if entry_point.node_id is None:
            self.nodes.add(node)
            entry_point.node_id = node.id
            self.repository.update(entry_point)
        else:
            node.id = entry_point.node_id
            self.nodes.update(node)

        return None

    def add(self, entry_point: EntryPoint) -> None:
        """
        Add entry point to list

        :param entry_point: EntryPoint instance
        :return:
        """
        self.repository.add(entry_point)

    def get(self, url: str) -> Optional[EntryPoint]:
        """
        Get entry point instance by url

        :param url: Url
        :return:
        """
        return self.repository.get(url)

    def update(self, entry_point: EntryPoint) -> None:
        """
        Update entry point in list

        :param entry_point: EntryPoint instance
        :return:
        """
        self.repository.update(entry_point)

    def list(
        self,
        offset: int = DEFAULT_LIST_OFFSET,
        limit: int = DEFAULT_LIST_LIMIT,
        sort_order: int = SORT_ORDER_ASCENDING,
        sort_property: str = PROPERTY_URL,
        node_id: UUID = None,
    ) -> List[EntryPoint]:
        """
        List entry points from repository

        :param offset: Offset index to get rows from
        :param limit: Number of rows to return
        :param sort_order: Sort order flag
        :param sort_property: Property flag to sort by
        :param node_id: Filter by node ID
        :return:
        """
        return self.repository.list(offset, limit, sort_order, sort_property, node_id)

    def delete(self, url: str) -> None:
        """
        Delete entry point from list

        :param url: Url of entry point
        :return:
        """
        # do not delete default entry points from config
        if url not in self.currencies.get_entry_point_urls():
            self.repository.delete(url)
            # switch current entry point to first in list
            self.set_current_url(self.repository.list(0, 1)[0].url)
            # set new entry point in preferences
            self.preferences_repository.set(
                CURRENT_ENTRY_POINT_URL, self.get_current_url()
            )
            entry_point = self.get(url)
            if entry_point is not None and entry_point.node_id is not None:
                self.nodes.delete(entry_point.node_id)

    def get_current_url(self) -> str:
        """
        Return current entry point url

        :return:
        """
        return self.current_url

    def set_current_url(self, url: str) -> None:
        """
        Set current entry point url

        :return:
        """
        self.current_url = url
        # update preference
        self.preferences_repository.set(
            CURRENT_ENTRY_POINT_URL,
            self.current_url,
        )
        # switch current connection
        self.connections.disconnect()
        entry_point = self.get(self.current_url)
        if entry_point is not None:
            self.connections.connect(entry_point)
            self.network_fetch_current_node()

    def _on_connections_connected(self, _: ConnectionsEvent):
        """
        Triggered when the connection is established

        :param _:
        :return:
        """
        self.network_fetch_current_node()



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import abc
from typing import List, Optional

from tikka.domains.entities.wallet import Wallet


class WalletsRepositoryInterface(abc.ABC):
    """
    AccountRepositoryInterface class
    """

    @abc.abstractmethod
    def list(self) -> List[Wallet]:
        """
        List wallets from repository

        :return:
        """
        raise NotImplementedError

    @abc.abstractmethod
    def add(self, wallet: Wallet) -> None:
        """
        Add a new wallet in repository

        :param wallet: Wallet instance
        :return:
        """
        raise NotImplementedError

    @abc.abstractmethod
    def get(self, address: str) -> Optional[Wallet]:
        """
        Return Wallet instance from repository

        :param address: Wallet address
        :return:
        """
        raise NotImplementedError

    def update(self, wallet: Wallet) -> None:
        """
        Update wallet in repository

        :param wallet: Wallet instance
        :return:
        """
        raise NotImplementedError

    @abc.abstractmethod
    def delete(self, address: str) -> None:
        """
        Delete wallet in repository

        :param address: Wallet address to delete
        :return:
        """
        raise NotImplementedError

    @abc.abstractmethod
    def exists(self, address: str) -> bool:
        """
        Return True if wallet with address is in repository, else False

        :param address: Wallet address to check
        :return:
        """
        raise NotImplementedError



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import abc
import json
import logging
import sqlite3
from pathlib import Path
from queue import Queue
from threading import Thread
from typing import Any, List, Optional, Union

from yoyo import get_backend, read_migrations

from tikka.domains.entities.constants import (
    DATABASE_FILE_EXTENSION,
    DATABASE_MIGRATIONS_PATH,
)


class Sqlite3Client(Thread):
    """
    Sqlite3 database thread client class

    """

    path: Path

    def __init__(self, connection_name: str, path: Path):
        """
        Init a Sqlite3 database client adapter instance as a thread

        :param connection_name: Name of connection
        :param path: Path to database data
        """
        super().__init__()

        self.path = path
        self.db_path = self.path.joinpath(
            f"{connection_name}{DATABASE_FILE_EXTENSION}"
        ).expanduser()

        # update database
        self.migrate()

        self.connection = None
        self.requests: Queue = Queue()
        self.start()

    def run(self):
        """
        Started asynchronously with Thread.start()

        :return:
        """
        connection = sqlite3.connect(str(self.db_path))
        cursor = connection.cursor()
        while True:
            request, arg, results = self.requests.get()
            if request == "--close--":
                break
            if not arg:
                try:
                    cursor.execute(request)
                except Exception as exception:
                    logging.exception(exception)
            else:
                try:
                    cursor.execute(request, arg)
                except Exception as exception:
                    logging.exception(exception)
            if results:
                for row in cursor:
                    results.put(row)
                results.put("--no more--")

            connection.commit()

        connection.close()
        logging.debug("Sqlite3 connection closed and thread terminated.")

    def execute(
        self,
        request: str,
        args: Optional[Union[List, tuple]] = None,
        result: Optional[Any] = None,
    ):
        """
        Execute SQL request with arg

        :param request: SQL request
        :param args: Arguments of SQL request
        :param result: Result to add to results
        :return:
        """
        self.requests.put((request, args or tuple(), result))

    def select(
        self, request: str, args: Optional[Union[List, tuple]] = None
    ) -> List[tuple]:
        """
        Execute Select request returning an iterator on result set

        :param request: SQL request
        :param args: Arguments of SQL request
        :return:
        """
        results: Queue = Queue()
        self.execute(request, args, results)
        rows = []
        while True:
            row = results.get()
            if row == "--no more--":
                break
            rows.append(row)

        return rows

    def close(self):
        """
        Close connection

        :return:
        """
        # Closing the connection
        self.execute("--close--")

    def insert(self, table: str, **kwargs):
        """
        Create a new entry in table, with field=value kwargs

        :param table: Table name
        :param kwargs: fields with their values
        :return:
        """
        fields_string = ",".join(kwargs.keys())
        values_string = ",".join(["?" for _ in range(len(kwargs))])
        filtered_values = []
        for value in kwargs.values():
            # serialize dict to json string
            filtered_values.append(
                json.dumps(value) if isinstance(value, dict) else value
            )

        sql = f"INSERT INTO {table} ({fields_string}) VALUES ({values_string})"
        self.execute(sql, filtered_values)

    def select_one(
        self, sql: str, args: Optional[Union[List, tuple]] = None
    ) -> Optional[tuple]:
        """
        Execute SELECT sql query and return first result

        :param sql: SELECT query
        :param args: Query arguments
        :return:
        """
        results = self.select(sql, args)
        if len(results) > 0:
            return results[0]

        return None

    def update(self, table: str, where: str, **kwargs):
        """
        Update rows of table selected by where from **kwargs

        :param table: Table name
        :param where: WHERE statement
        :param kwargs: field=values kwargs
        :return:
        """
        set_statement = ",".join([f"{field}=?" for field in kwargs])

        sql = f"UPDATE {table} SET {set_statement} WHERE {where}"
        values = list(kwargs.values())

        self._update(sql, values)

    def _update(self, sql: str, values: list):
        """
        Send update request sql with values

        :param sql: SQL query
        :param values: Values to inject as sql params
        :return:
        """
        filtered_values = []
        for value in values:
            # serialize dict to json string
            filtered_values.append(
                json.dumps(value) if isinstance(value, dict) else value
            )
        self.execute(sql, filtered_values)

    def delete(self, table: str, **kwargs):
        """
        Delete a row from table where key=value (AND) from kwargs

        :param table: Table to delete from
        :param kwargs: Key/Value conditions (AND)
        :return:
        """
        conditions = " AND ".join([f"{key}=?" for key in kwargs])

        sql = f"DELETE FROM {table} WHERE {conditions}"
        self.execute(sql, list(kwargs.values()))

    def clear(self, table: str):
        """
        Clear table entries

        :param table: Name of the table
        :return:
        """
        # delete all entries
        self.execute(f"DELETE FROM {table}")

    def migrate(self):
        """
        Use Yoyo Python library to handle current database migrations

        :return:
        """
        migrations_path = str(
            Path(__file__).parent.parent.joinpath(DATABASE_MIGRATIONS_PATH).expanduser()
        )
        migrations = read_migrations(migrations_path)

        backend = get_backend("sqlite:///" + str(self.db_path))
        with backend.lock():
            # Apply any outstanding migrations
            backend.apply_migrations(backend.to_apply(migrations))
            logging.debug(backend.applied_migrations_sql)


class NoConnectionError(Exception):
    pass


class Sqlite3RepositoryInterface(abc.ABC):
    """
    Sqlite3RepositoryInterface class
    """

    def __init__(self, client: Sqlite3Client) -> None:
        """
        Init Sqlite3RepositoryInterface inheritor with sqlite3 client

        :param client: Sqlite3Client instance
        :return:
        """
        self.client = client

    def set_client(self, client: Sqlite3Client):
        """
        Set repository client to client

        :param client: Sqlite3Client instance
        :return:
        """
        self.client = client



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import sys
from typing import Optional

from PyQt5 import QtCore, QtGui
from PyQt5.QtCore import QPoint, QSize
from PyQt5.QtWidgets import (
    QApplication,
    QHBoxLayout,
    QLabel,
    QMainWindow,
    QTableWidgetItem,
    QWidget,
)

from tikka.domains.application import Application
from tikka.domains.entities.account import Account
from tikka.domains.entities.constants import DATA_PATH
from tikka.domains.entities.events import AccountEvent, CurrencyEvent
from tikka.slots.pyqt.entities.constants import (
    ICON_ACCOUNT_LOCKED,
    ICON_ACCOUNT_UNLOCKED,
)
from tikka.slots.pyqt.resources.gui.widgets.account_list_rc import Ui_AccountListWidget
from tikka.slots.pyqt.widgets.account_menu import AccountPopupMenu


class CenteredIconWidget(QWidget):
    """
    CenteredIconWidget class
    """

    def __init__(self, icon_path: str):
        """
        Init a widget with the icon centered

        :param icon_path: Path of the icon in resource index
        """
        super().__init__()
        # create a label with the icon as bitmap
        label = QLabel()
        label.setPixmap(QtGui.QPixmap(icon_path))
        label.setScaledContents(True)
        icon_size = QSize(16, 16)
        label.setMaximumSize(icon_size)

        # create a layout to center icon
        layout = QHBoxLayout(self)
        layout.addWidget(label, 0, QtCore.Qt.AlignCenter)
        layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(layout)


class AccountListWidget(QWidget, Ui_AccountListWidget):
    """
    AccountListWidget class
    """

    def __init__(
        self,
        application: Application,
        parent: Optional[QWidget] = None,
    ) -> None:
        """
        Init AccountListWidget instance

        :param application: Application instance
        :param parent: QWidget instance
        """
        super().__init__(parent=parent)
        self.setupUi(self)

        self.application = application

        # set the table rows
        self.init_table_rows()

        # table events
        self.tableWidget.customContextMenuRequested.connect(self.on_context_menu)

        self.application.event_dispatcher.add_event_listener(
            CurrencyEvent.EVENT_TYPE_CHANGED, self.on_currency_event
        )
        self.application.event_dispatcher.add_event_listener(
            AccountEvent.EVENT_TYPE_ADD, self.on_add_account_event
        )
        self.application.event_dispatcher.add_event_listener(
            AccountEvent.EVENT_TYPE_DELETE, self.on_delete_account_event
        )
        self.application.event_dispatcher.add_event_listener(
            AccountEvent.EVENT_TYPE_UPDATE, self.on_update_account_event
        )

    def init_table_rows(self):
        """
        Init table rows with values

        :return:
        """
        # remove all rows
        for _ in range(0, self.tableWidget.rowCount()):
            self.tableWidget.removeRow(0)

        for account in self.application.accounts.list:
            self._add_row(account)

    def _add_row(self, account: Account) -> None:
        """
        Add a row in list for account

        :param account: Account instance

        :return:
        """
        index = self.tableWidget.rowCount()
        self.tableWidget.insertRow(index)
        self._set_row(index, account)

    def _set_row(self, index: int, account: Account):
        """
        Populate row from account data

        :param account: Account instance
        :return:
        """
        if self.application.wallets.exists(account.address):
            if account.keypair is None:
                widget = CenteredIconWidget(ICON_ACCOUNT_LOCKED)
                self.tableWidget.setCellWidget(index, 0, widget)
            else:
                widget = CenteredIconWidget(ICON_ACCOUNT_UNLOCKED)
                self.tableWidget.setCellWidget(index, 0, widget)
        else:
            self.tableWidget.removeCellWidget(index, 0)

        self.tableWidget.setItem(
            index,
            1,
            QTableWidgetItem(account.address),
        )

    def on_currency_event(self, _):
        """
        When a currency event is triggered

        :return:
        """
        self.init_table_rows()

    def on_add_account_event(self, event: AccountEvent):
        """
        Add account row when account is created

        :param event: AccountEvent instance
        :return:
        """
        self._add_row(event.account)

    def on_delete_account_event(self, event: AccountEvent):
        """
        Remove account row when account is deleted

        :param event: AccountEvent instance
        :return:
        """
        for index in range(0, self.tableWidget.rowCount()):
            if (
                self.tableWidget.item(index, 1) is not None
                and self.tableWidget.item(index, 1).text() == event.account.address
            ):
                self.tableWidget.removeRow(index)

    def on_update_account_event(self, event: AccountEvent):
        """
        Update account row when account is updated

        :param event: AccountEvent instance
        :return:
        """
        for index in range(0, self.tableWidget.rowCount()):
            if self.tableWidget.item(index, 1).text() == event.account.address:
                self._set_row(index, event.account)

    def on_context_menu(self, position: QPoint):
        """
        When right button on table widget

        :param position: QPoint instance
        :return:
        """
        # get selected account
        account = self.application.accounts.get_by_index(self.tableWidget.currentRow())
        # display popup menu at click position
        AccountPopupMenu(self.application, account).exec_(
            self.tableWidget.mapToGlobal(position)
        )


if __name__ == "__main__":
    qapp = QApplication(sys.argv)

    application_ = Application(DATA_PATH)

    main_window = QMainWindow()
    main_window.show()

    main_window.setCentralWidget(AccountListWidget(application_, main_window))

    sys.exit(qapp.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/account_import.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AccountImportDialog(object):
    def setupUi(self, AccountImportDialog):
        AccountImportDialog.setObjectName("AccountImportDialog")
        AccountImportDialog.resize(961, 400)
        self.verticalLayout = QtWidgets.QVBoxLayout(AccountImportDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.imagelabel = QtWidgets.QLabel(AccountImportDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.imagelabel.sizePolicy().hasHeightForWidth())
        self.imagelabel.setSizePolicy(sizePolicy)
        self.imagelabel.setMinimumSize(QtCore.QSize(0, 0))
        self.imagelabel.setMaximumSize(QtCore.QSize(200, 200))
        self.imagelabel.setSizeIncrement(QtCore.QSize(0, 0))
        self.imagelabel.setBaseSize(QtCore.QSize(0, 0))
        self.imagelabel.setText("")
        self.imagelabel.setPixmap(QtGui.QPixmap(":/images/safe_background"))
        self.imagelabel.setScaledContents(True)
        self.imagelabel.setAlignment(QtCore.Qt.AlignCenter)
        self.imagelabel.setObjectName("imagelabel")
        self.verticalLayout.addWidget(self.imagelabel, 0, QtCore.Qt.AlignHCenter)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.mnemonicLabel = QtWidgets.QLabel(AccountImportDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.mnemonicLabel.setFont(font)
        self.mnemonicLabel.setObjectName("mnemonicLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.mnemonicLabel)
        self.mnemonicLineEdit = QtWidgets.QLineEdit(AccountImportDialog)
        self.mnemonicLineEdit.setReadOnly(False)
        self.mnemonicLineEdit.setObjectName("mnemonicLineEdit")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.mnemonicLineEdit)
        self.mnemonicLanguageLabel = QtWidgets.QLabel(AccountImportDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setItalic(False)
        font.setWeight(75)
        self.mnemonicLanguageLabel.setFont(font)
        self.mnemonicLanguageLabel.setObjectName("mnemonicLanguageLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.mnemonicLanguageLabel)
        self.mnemonicLanguageComboBox = QtWidgets.QComboBox(AccountImportDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mnemonicLanguageComboBox.sizePolicy().hasHeightForWidth())
        self.mnemonicLanguageComboBox.setSizePolicy(sizePolicy)
        self.mnemonicLanguageComboBox.setObjectName("mnemonicLanguageComboBox")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.mnemonicLanguageComboBox)
        self.addressLabel = QtWidgets.QLabel(AccountImportDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.addressLabel.setFont(font)
        self.addressLabel.setObjectName("addressLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.addressLabel)
        self.addressValueLabel = QtWidgets.QLabel(AccountImportDialog)
        self.addressValueLabel.setText("")
        self.addressValueLabel.setObjectName("addressValueLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.addressValueLabel)
        self.verticalLayout.addLayout(self.formLayout)
        self.errorLabel = QtWidgets.QLabel(AccountImportDialog)
        self.errorLabel.setStyleSheet("color: red")
        self.errorLabel.setText("")
        self.errorLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.errorLabel.setObjectName("errorLabel")
        self.verticalLayout.addWidget(self.errorLabel)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.passwordLabel = QtWidgets.QLabel(AccountImportDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.passwordLabel.sizePolicy().hasHeightForWidth())
        self.passwordLabel.setSizePolicy(sizePolicy)
        self.passwordLabel.setSizeIncrement(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.passwordLabel.setFont(font)
        self.passwordLabel.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.passwordLabel.setObjectName("passwordLabel")
        self.horizontalLayout.addWidget(self.passwordLabel)
        self.passwordLineEdit = QtWidgets.QLineEdit(AccountImportDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.passwordLineEdit.sizePolicy().hasHeightForWidth())
        self.passwordLineEdit.setSizePolicy(sizePolicy)
        self.passwordLineEdit.setMaxLength(20)
        self.passwordLineEdit.setAlignment(QtCore.Qt.AlignCenter)
        self.passwordLineEdit.setReadOnly(True)
        self.passwordLineEdit.setObjectName("passwordLineEdit")
        self.horizontalLayout.addWidget(self.passwordLineEdit)
        self.passwordChangeButton = QtWidgets.QPushButton(AccountImportDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.passwordChangeButton.sizePolicy().hasHeightForWidth())
        self.passwordChangeButton.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/refresh_icon"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.passwordChangeButton.setIcon(icon)
        self.passwordChangeButton.setObjectName("passwordChangeButton")
        self.horizontalLayout.addWidget(self.passwordChangeButton)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.buttonBox = QtWidgets.QDialogButtonBox(AccountImportDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox, 0, QtCore.Qt.AlignHCenter)

        self.retranslateUi(AccountImportDialog)
        self.buttonBox.accepted.connect(AccountImportDialog.accept) # type: ignore
        self.buttonBox.rejected.connect(AccountImportDialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(AccountImportDialog)

    def retranslateUi(self, AccountImportDialog):
        #  _translate = QtCore.QCoreApplication.translate
        AccountImportDialog.setWindowTitle(_("Import account"))
        self.mnemonicLabel.setText(_("Mnemonic"))
        self.mnemonicLanguageLabel.setText(_("Mnemonic language"))
        self.addressLabel.setText(_("Address"))
        self.passwordLabel.setText(_("Password"))
        self.passwordChangeButton.setText(_("Change"))
import tikka.slots.pyqt.resources.icons.index_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/transfer.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_TransferDialog(object):
    def setupUi(self, TransferDialog):
        TransferDialog.setObjectName("TransferDialog")
        TransferDialog.resize(596, 167)
        self.gridLayout = QtWidgets.QGridLayout(TransferDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.sendButton = QtWidgets.QPushButton(TransferDialog)
        self.sendButton.setObjectName("sendButton")
        self.gridLayout.addWidget(self.sendButton, 3, 3, 1, 1)
        self.feesButton = QtWidgets.QPushButton(TransferDialog)
        self.feesButton.setObjectName("feesButton")
        self.gridLayout.addWidget(self.feesButton, 2, 3, 1, 1)
        self.feesLabel = QtWidgets.QLabel(TransferDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.feesLabel.setFont(font)
        self.feesLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.feesLabel.setObjectName("feesLabel")
        self.gridLayout.addWidget(self.feesLabel, 3, 0, 1, 1)
        self.amountUnitComboBox = QtWidgets.QComboBox(TransferDialog)
        self.amountUnitComboBox.setObjectName("amountUnitComboBox")
        self.gridLayout.addWidget(self.amountUnitComboBox, 2, 2, 1, 1)
        self.senderBalanceValueLabel = QtWidgets.QLabel(TransferDialog)
        self.senderBalanceValueLabel.setText("")
        self.senderBalanceValueLabel.setObjectName("senderBalanceValueLabel")
        self.gridLayout.addWidget(self.senderBalanceValueLabel, 0, 8, 1, 1)
        self.recipientBalanceValueLabel = QtWidgets.QLabel(TransferDialog)
        self.recipientBalanceValueLabel.setText("")
        self.recipientBalanceValueLabel.setObjectName("recipientBalanceValueLabel")
        self.gridLayout.addWidget(self.recipientBalanceValueLabel, 1, 8, 1, 1)
        self.amountLabel = QtWidgets.QLabel(TransferDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.amountLabel.setFont(font)
        self.amountLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.amountLabel.setObjectName("amountLabel")
        self.gridLayout.addWidget(self.amountLabel, 2, 0, 1, 1)
        self.amountDoubleSpinBox = QtWidgets.QDoubleSpinBox(TransferDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.amountDoubleSpinBox.sizePolicy().hasHeightForWidth())
        self.amountDoubleSpinBox.setSizePolicy(sizePolicy)
        self.amountDoubleSpinBox.setObjectName("amountDoubleSpinBox")
        self.gridLayout.addWidget(self.amountDoubleSpinBox, 2, 1, 1, 1)
        self.recipientLabel = QtWidgets.QLabel(TransferDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.recipientLabel.setFont(font)
        self.recipientLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.recipientLabel.setObjectName("recipientLabel")
        self.gridLayout.addWidget(self.recipientLabel, 1, 0, 1, 1)
        self.recipientAddressLineEdit = QtWidgets.QLineEdit(TransferDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.recipientAddressLineEdit.sizePolicy().hasHeightForWidth())
        self.recipientAddressLineEdit.setSizePolicy(sizePolicy)
        self.recipientAddressLineEdit.setObjectName("recipientAddressLineEdit")
        self.gridLayout.addWidget(self.recipientAddressLineEdit, 1, 1, 1, 4)
        self.sendStatusLabel = QtWidgets.QLabel(TransferDialog)
        self.sendStatusLabel.setStyleSheet("")
        self.sendStatusLabel.setText("")
        self.sendStatusLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.sendStatusLabel.setObjectName("sendStatusLabel")
        self.gridLayout.addWidget(self.sendStatusLabel, 4, 1, 1, 4)
        self.senderLabel = QtWidgets.QLabel(TransferDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.senderLabel.setFont(font)
        self.senderLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.senderLabel.setObjectName("senderLabel")
        self.gridLayout.addWidget(self.senderLabel, 0, 0, 1, 1)
        self.senderAddressValueLabel = QtWidgets.QLabel(TransferDialog)
        self.senderAddressValueLabel.setText("")
        self.senderAddressValueLabel.setObjectName("senderAddressValueLabel")
        self.gridLayout.addWidget(self.senderAddressValueLabel, 0, 1, 1, 4)
        self.buttonBox = QtWidgets.QDialogButtonBox(TransferDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.buttonBox.sizePolicy().hasHeightForWidth())
        self.buttonBox.setSizePolicy(sizePolicy)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Close)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 4, 8, 1, 1, QtCore.Qt.AlignHCenter)
        self.loaderIconLabel = QtWidgets.QLabel(TransferDialog)
        self.loaderIconLabel.setEnabled(True)
        self.loaderIconLabel.setMaximumSize(QtCore.QSize(16, 16))
        self.loaderIconLabel.setText("")
        self.loaderIconLabel.setScaledContents(True)
        self.loaderIconLabel.setObjectName("loaderIconLabel")
        self.gridLayout.addWidget(self.loaderIconLabel, 2, 5, 1, 1)
        self.feesValueLabel = QtWidgets.QLabel(TransferDialog)
        self.feesValueLabel.setText("")
        self.feesValueLabel.setObjectName("feesValueLabel")
        self.gridLayout.addWidget(self.feesValueLabel, 3, 1, 1, 1)

        self.retranslateUi(TransferDialog)
        QtCore.QMetaObject.connectSlotsByName(TransferDialog)

    def retranslateUi(self, TransferDialog):
        #  _translate = QtCore.QCoreApplication.translate
        TransferDialog.setWindowTitle(_("Transfer"))
        self.sendButton.setText(_("Send"))
        self.feesButton.setText(_("Fees"))
        self.feesLabel.setText(_("Fees"))
        self.amountLabel.setText(_("Amount"))
        self.recipientLabel.setText(_("Recipient"))
        self.senderLabel.setText(_("Sender"))
import tikka.slots.pyqt.resources.icons.index_rc



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tikka',
 'tikka.adapters',
 'tikka.adapters.network',
 'tikka.adapters.network.rpc',
 'tikka.adapters.repository',
 'tikka.domains',
 'tikka.domains.entities',
 'tikka.interfaces',
 'tikka.interfaces.adapters',
 'tikka.interfaces.adapters.network',
 'tikka.interfaces.adapters.repository',
 'tikka.interfaces.domains',
 'tikka.interfaces.entities',
 'tikka.libs',
 'tikka.slots',
 'tikka.slots.pyqt',
 'tikka.slots.pyqt.entities',
 'tikka.slots.pyqt.models',
 'tikka.slots.pyqt.resources',
 'tikka.slots.pyqt.resources.gui',
 'tikka.slots.pyqt.resources.gui.widgets',
 'tikka.slots.pyqt.resources.gui.windows',
 'tikka.slots.pyqt.resources.icons',
 'tikka.slots.pyqt.widgets',
 'tikka.slots.pyqt.windows']

package_data = \
{'': ['*'],
 'tikka': ['locales/en_US/*',
           'locales/en_US/LC_MESSAGES/*',
           'locales/fr_FR/*',
           'locales/fr_FR/LC_MESSAGES/*'],
 'tikka.adapters.repository': ['assets/*', 'assets/migrations/*']}

install_requires = \
['PyQt5>=5.15.6,<6.0.0',
 'PyYAML>=6.0,<7.0',
 'duniterpy==1.0.0',
 'mnemonic>=0.19,<0.20',
 'substrate-interface>=1.2.4,<2.0.0',
 'yoyo-migrations>=7.3.1,<8.0.0']

entry_points = \
{'console_scripts': ['tikka = tikka.__main__:main']}

setup_kwargs = {
    'name': 'tikka',
    'version': '0.5.2',
    'description': 'Tikka is a fast and light Python/Tk client to manage your Ğ1 accounts',
    'long_description': None,
    'author': 'Vincent Texier',
    'author_email': 'vit@free.fr',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.0,<4.0.0',
}


setup(**setup_kwargs)



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import abc

from tikka.adapters.network.rpc.connection import RPCConnection
from tikka.domains.entities.entry_point import EntryPoint
from tikka.domains.events import EventDispatcher


class ConnectionsInterface(abc.ABC):
    """
    ConnectionsInterface class
    """

    def __init__(self, rpc: RPCConnection, event_dispatcher: EventDispatcher) -> None:
        """
        Init Connections instance with dependencies

        :param rpc: RPCConnection instance
        :param event_dispatcher: EventDispatcher instance
        """
        self.rpc = rpc
        self.event_dispatcher = event_dispatcher

    @abc.abstractmethod
    def connect(self, entry_point: EntryPoint) -> None:
        """
        Connect all connections

        :return:
        """
        raise NotImplementedError

    @abc.abstractmethod
    def disconnect(self):
        """
        Disconnect all connections

        :return:
        """
        raise NotImplementedError

    @abc.abstractmethod
    def is_connected(self) -> bool:
        """
        Return True if connection is active, False otherwise

        :return:
        """
        raise NotImplementedError



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import logging
from typing import Optional

from tikka.interfaces.adapters.network.currency import NetworkCurrencyInterface


class NetworkCurrency(NetworkCurrencyInterface):
    """
    NetworkCurrency class
    """

    def get_token_decimals(self) -> Optional[int]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NetworkCurrencyInterface.get_token_decimals.__doc__
        )
        if not self.connections.is_connected():
            return None
        if self.connections.rpc.client is None:
            return None

        try:
            result = self.connections.rpc.client.rpc_request(
                "system_properties", []
            ).get("result")
        except Exception as exception:
            logging.exception(exception)
            return None

        return result["tokenDecimals"]

    def get_token_symbol(self) -> Optional[str]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NetworkCurrencyInterface.get_token_symbol.__doc__
        )
        if not self.connections.is_connected():
            return None
        if self.connections.rpc.client is None:
            return None

        try:
            result = self.connections.rpc.client.rpc_request(
                "system_properties", []
            ).get("result")
        except Exception as exception:
            logging.exception(exception)
            return None

        return result["tokenSymbol"]

    def get_universal_dividend(self) -> Optional[int]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NetworkCurrencyInterface.get_universal_dividend.__doc__
        )
        if not self.connections.is_connected():
            return None
        if self.connections.rpc.client is None:
            return None

        try:
            result = self.connections.rpc.client.query("UniversalDividend", "CurrentUd")
        except Exception as exception:
            logging.exception(exception)
            return None

        return result.value

    def get_monetary_mass(self) -> Optional[int]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NetworkCurrencyInterface.get_monetary_mass.__doc__
        )
        if not self.connections.is_connected():
            return None
        if self.connections.rpc.client is None:
            return None

        try:
            result = self.connections.rpc.client.query(
                "UniversalDividend", "MonetaryMass"
            )
        except Exception as exception:
            logging.exception(exception)
            return None

        return result.value

    def get_members_count(self) -> Optional[int]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NetworkCurrencyInterface.get_members_count.__doc__
        )
        if not self.connections.is_connected():
            return None
        if self.connections.rpc.client is None:
            return None

        try:
            result = self.connections.rpc.client.query(
                "UdAccountsStorage", "CounterForUdAccounts"
            )
        except Exception as exception:
            logging.exception(exception)
            return None

        return result.value
